if(!dojo._hasResource["com.ibm.data.XpathHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.XpathHelper"] = true;
dojo.provide( "com.ibm.data.XpathHelper" );
// summary: This module is meant to be extended by modules that want to use XPath
//		to query DOM documents for information.  It provides read support and limited 
// 		write support.  Write support is limited in that if an xpath query is used to
//		set a node, property of a node, or DOM attribute of a node, then the write support
//		will only work if a node that matches that query already exists and is returned.
//		If no node matches the query and the Xpath query returns null, the all write operations
//		provided by this module will return false to indicate that the caller needs to handle
//		them in a different way like creating the node from scratch.
//
//		This module provides cross-browser methods for reading and writing to DOM nodes,
//		properties of DOM nodes, and DOM attributes of DOM nodes.  It also supports extensions
//		so that special constructs can be returned by a query instead of a DOM node or String value
//		of a property or attribute.  By default, no constructs are defined by this module.


dojo.require( "com.ibm.portal.xpath" );
dojo.require( "com.ibm.portal.xslt" );
dojo.require( "com.ibm.domUtilities" );

dojo.declare( "com.ibm.data.XpathHelper",
			null,
			{
				// namespaces: A map of prefix - namespace URI pairs used to specify what namespaces
				//		are used by this XpathHelper to query nodes in a DOM document
				namespaces: {},
				
				_getNodesByXpath: function(/*Object*/ context, /*String*/expr, /*String?*/prop) {
					// summary: Gets an array of DOM nodes or String node properties using the XPath expr relative 
					// 		to the context (not relative to the document root itself).  Returns an object allowing
					//		the caller to determine if the actual value requested is null or if there were no nodes
					//		found at all.
					// context: The XML element to use as the root context for the XPath query
					// expr: XPath expression used to find the nodes
					// prop: Optional prop to specify to get a property of the nodes instead of the nodes themselves.
					// returns: Object with two properties:
					//		-	{
					//		-		found: Boolean indicating if any nodes were found matching the XPath expression
					//		-		value: Array the actual nodes found by the expression, or the properties of the nodes
					// 		-			found by the expression as specified by the prop argument
					// 		-	} 
					var results = com.ibm.portal.xpath.evaluateXPath(expr, context, this.namespaces);
					var found = results && results.length > 0;
					if(prop) {
						var nodes = results;
						results = [];
						for(var i = 0; i < nodes.length; i++) {
							results[i] = this._getNodeProp(nodes[i], prop);
						}
					}
					return {found: found, value: results};
				},
				_getNodeByXpath: function(/*Object*/ context, /*String*/expr, /*String?*/ prop) {
					// summary: Gets a node using the XPath expr relative to 
					// 		the context	(not relative to the document root itself).  Returns an object allowing
					//		the caller to determine if the actual value requested is null or if there were no nodes
					//		found at all.
					// context: The XML element to use as the root context for the XPath query
					// expr: XPath expression used to find the node
					// prop: Optional prop to specify to get an property of the node instead of the node itself.
					// returns: Object with two properties:
					//		-	{
					//		-		found: Boolean indicating if any nodes were found matching the XPath expression
					//		-		value: DOMNode|Object the first node found by the expression, or the property of the first
					// 		-			node found by the expression as specified by the prop argument
					// 		-	}
					var result = this._getNodesByXpath(context, expr);
					var found = result.found;
					if(!found) return {found: found};
					var nodes = result.value;
					if(nodes.length > 0) {
						if(prop) return {found: true, value: this._getNodeProp(nodes[0], prop)};
						else return {found: true, value: nodes[0]};
					}
					return {found: false};
				},
				_getNodeProp: function(/*DOMNode*/ node, /*String*/ prop) {
					// summary:  Function to provide cross-browser support for properties of a queried
					//		node.  If the prop string is not special, control will be passed to the 
					//		handleConstructs function for this object.
					//		Currently only supports textContent as a special property since not
					//		all browsers implement the textContent property of DOM nodes.
					switch(prop) {
						case "textContent":
							return dojox.data.dom.textContent(node);
						default:
							return this.handleConstructs(node, prop);
					}
				},
				handleConstructs: function(/*DOMNode*/ node, /*String*/ prop) {
					// summary: Allows a data store to return complex javascript objects
					//		or alternative values when a specific non-standard property of a node 
					// 		is requested.  Example: An Atom Feed data store may create a special 
					// 		Person Construct when querying an <atom:author> element.  This construct
					//		may provide accessor functions to the name, email, and URI of the author.
					//		See http://tools.ietf.org/html/rfc4287#section-3.2
					//		Constructs that are created here should implement and handle write functionality
					//		themselves as this module will not handle writes to constructs natively.
					
					return node[prop];
				},
				_setNodeProp: function(/*DOMNode*/ node, /*String*/ prop, /*Object*/ value) {
					// summary: Allows a data store to set properties of a node in a cross-browser
					//		way for properties that may be non-standard.
					//		Currently only supports textContent as a special property since not
					//		all browsers implement the textContent property of DOM nodes.
					//		Note that for special constructs that are created using the handleConstructs
					// 		function, write support is handled by the returned construct object not by
					//		this module's functions.
					switch(prop) {
						case "textContent":
							dojox.data.dom.textContent(node, value);
							break;
						default:
							node[prop] = value;
					}
				},
				_getAttrsByXpath: function(/*Object*/ context, /*String*/ expr, /*String*/attrName) {
					// summary: Gets an array of attribute values for the nodes returned 
					// 		using the XPath expr relative to the context (not relative to the 
					// 		document root itself).  The attributes are retrieved using the 
					// 		getAttributeNS/getAttribute	functions of the nodes.  Returns an object allowing
					//		the caller to determine if the actual value requested is null or if there were no nodes
					//		found at all.
					// context: The XML element to use as the root context for the XPath query
					// expr: XPath expression used to find the nodes
					// attrName: The name of the attribute to get from each node returned by the XPath expr.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					// returns: Object with two properties:
					//		-	{
					//		-		found: Boolean indicating if any nodes were found matching the XPath expression
					//		-		value: Array the attribute values of the nodes found by the expression
					// 		-	}
					var result = this._getNodesByXpath(context, expr);
					var found = result.found;
					if(!found) return {found: found};
					var nodes = result.value;
					var attrs = [];
					for(var i = 0; i < nodes.length; i++) {
						attrs[i] = this._getAttribute(nodes[i], attrName);
					}
                    return {found: found, value: attrs};
				},
				_getAttrByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ attrName) {
					// summary: Gets the attribute value of the first node returned 
					// 		using the XPath expr relative to the context (not relative to the 
					// 		document root itself).  The attribute is retrieved using the 
					// 		getAttributeNS/getAttribute	functions of the node.  Returns an object allowing
					//		the caller to determine if the actual value requested is null or if there were no nodes
					//		found at all.
					// context: The XML element to use as the root context for the XPath query
					// expr: XPath expression used to find the node
					// attrName: The name of the attribute to get from the node returned by the XPath expr.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					// returns: Object with two properties:
					//		-	{
					//		-		found: Boolean indicating if any nodes were found matching the XPath expression
					//		-		value: Array the attribute value of the first node found by the expression
					// 		-	}
					var result = this._getNodeByXpath(context, expr);
					var found = result.found;
					if(!found) return {found: found}; 
					var node = result.value;
                    return {found: found, value: this._getAttribute(node, attrName)};
				},
				_extractNSInfo: function(str) {
					// summary: Extracts the namespace URI, prefix, and local name of a QName according to this
					//		object's namespaces map that includes known prefixes and their namespace URIs.  If
					//		the string is not namespaced, then the namespaceURI and prefix are set to null, and
					//		the localName is set to the str as-is.
					// str: QName to extract namespace from
					// returns: Map - {namespaceURI: namespaceURI, prefix: prefix, localName: localName}
					return com.ibm.domUtilities.nsInfo(str, this.namespaces);
				},
				_getAttribute: function(/*DOMNode*/ node, /*String*/ attrName) {
					// summary: Gets an attribute of the given node, supporting namespaced and 
					// 		non-namespaced attributes.
					// node: Node to retrieve the attribute value from
					// attrName: The name of the attribute to get from the node.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					return com.ibm.domUtilities.getAttribute(node, attrName, this.namespaces);
				},
				_setAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*String*/ value) {
					// summary: Sets an attribute of the given node, supporting namespaced and 
					// 		non-namespaced attributes.
					// node: Node to set the attribute value on
					// attrName: The name of the attribute to set on the node.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					// value: The actual value to store in the node at the key attribute specified by attrName.
					if(value) com.ibm.domUtilities.setAttribute(node, attrName, value, this.namespaces);
					else com.ibm.domUtilities.removeAttribute(node, attrName, this.namespaces);
				},
				_setNodesByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ prop, /*Array*/values) {
					// Empty for now since there's little use for this type of function, particularly due to 
					// mismatches between the number of values and number of returned nodes.
					//var nodes = this._getNodesByXpath(context, expr);
					
					return false;
				},
				_setNodeByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ prop, /*Object*/value) {
					// summary: Will replace a node returned by the xpath expr query with the value object
					// 		if prop is null.  If prop exists, the specified property on the returned node
					// 		will be set to the value.
					// 		If no node is found that matches the xpath expr, this function returns false to indicate
					//		failure to the caller for further processing if necessary.
					// context: The DOM element to use as the root context for the XPath query
					// expr: XPath expression used to find the node
					// prop: Optional prop to specify to set a property of the node instead of the node itself.
					// returns: true if the set operation was successful, false otherwise
					var result = this._getNodeByXpath(context, expr);
					var found = result.found;
					if(!found) return false; 
					var node = result.value;
					if(prop) {
						this._setNodeProp(node, prop, value);
					}
					else {
						dojo.place(value, node, "before");
						node.parentNode.removeChild(node);
						delete node;
					}
					return true;
				},
				_setAttrsByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ attrName, /*Array*/values) {
					// Empty for now since there's little use for this type of function, particularly due to 
					// mismatches between the number of values and number of returned nodes.
					//var nodes = this._getNodesByXpath(context, expr);
					
					return false;
				},
				_setAttrByXpath: function(/*Object*/ context, /*String*/expr, /*String*/ attrName, /*String*/value) {
					// summary: The specified DOM attribute on the returned node will be set to the value.
					// 		If no node is found that matches the xpath expr, this function returns false to indicate
					//		failure to the caller for further processing if necessary.
					// context: The DOM element to use as the root context for the XPath query
					// expr: XPath expression used to find the node
					// attrName: The name of the attribute to get from the node returned by the XPath expr.
					//		If the attribute is namespaced, it will be split into its parts using the : delimiter
					//		and the namespace used during lookup will be the one keyed in this item or store's 
					// 		namespace map.
					// returns: true if the set operation was successful, false otherwise
					var result = this._getNodeByXpath(context, expr);
					var found = result.found;
					if(!found) return false;
					var node = result.value;
					this._setAttribute(node, attrName, value);
					return true;
				},
				_createNSElement: function(doc, tagName, attrMap) {
					// summary: Cross-browser implementation for creating a namespaced node using the namespace
					//		map in this object to resolve prefixes to namespace URIs.
					//		If the tagName is not namespaced or a namespace URI cannot be resolved
					// 		using this object's namespaces map, a non-namespaced element will be created instead.
					// doc: DOMDoc document object to use to create the DOM node.
					// tagName: String name for the tag name of the element to create.  
					// 		Can be namespaced like prefix:localName or not like localName.
					// attrMap: Optional argument specifying a key-value map of attributes to be
					// 		set on the node before returning it. 
					// returns: DOMNode A new DOM node that was created by doc
					return com.ibm.domUtilities.createElement(doc, tagName, this.namespaces, attrMap);
				},
				__getClassName: function() {
					var cls = this.declaredClass;
					if(!cls) return "Object";
					var idx = cls.lastIndexOf(".");
					if(idx > -1 && (idx < cls.length - 1)) cls = cls.substring(idx + 1);
					return cls;
				}
			}
);

dojo.declare( "com.ibm.data._XmlConstruct",
			com.ibm.data.XpathHelper,
			{
				// summary: Provides a base construct for modules to extend when building constructs
				//		to be returned by queries using this module's APIs.
				isConstruct: true,
				toString: function() {
					// summary: Default toString method for this object so that it can be inserted
					//		in a string and be properly represented.  Overridable by extensions of this
					//		module.
					return "<"   + this.__getClassName() + ">";
				}
			}
);


}
