if(!dojo._hasResource["com.ibm.mashups.builder.model.ContextMenu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mashups.builder.model.ContextMenu"] = true;
dojo.provide("com.ibm.mashups.builder.model.ContextMenu" );

//<INTERFACE>
dojo.require("com.ibm.mashups.builder.model.ContextMenuLoader");

/**
 * This interface defines Context Menu operations.
 * @ibm-module Builder
 */
dojo.declare( "com.ibm.mashups.builder.model.ContextMenu", null, {

	/**
	 * Returns a array of JSON objects for a given collection of menu choices for the selected context menu. <br/>
	 * Each object in the array follows the &nbsp;{@link com.ibm.mashups.builder.model.ContextMenuItem ContextMenuItem} 
	 * object format plus any custom extension properties that a context menu may define.<br/>
	 * @ibm-api
	 * @param {String} contextMenuId  this is id of the context menu.  Examples: ContextMenu.page, ContextMenu.space, ContextMenu.toolbox, ContextMenu.modeSelector ContextMenu.widgetSkin, or any additional extension.  Must not be <code>null</code>.
	 * @param {String} targetResourceId this is the ID of the resource to associate the decision for this menu choice with.  This will be passed into the respective contextmenu functions and must not be <code>null</code>.  
	 * @param {String} targetResourceType this is the resource type of the resource to associate the decision for this menu choice with.  This will be passed into the respective contextmenu functions.  May be <code>null</code>.
	 * Possible values for this could be "com.ibm.mashups.enabler.space.SpaceNode", "com.ibm.mashups.iwidget.widget.IWidgetDefinition", "com.ibm.mashups.enabler.navigation.NavigationNode",
	 * or any other type used by an extension.  
	 * @param {Object} someObject this allows an arbitrary object to be passed into the context menu items' functions in case they need additional data for making the decisions.
	 * @type JSON[]
	 * @return {JSON[]} A JSON object array of menu choices; may be an empty array if no choices exist for that context menu, but never <code>null</code>.
	 */
	getContextMenu:function(/*String*/ contextMenuId, /*String*/ targetResourceId, /*String*/ targetResourceType, /*Object*/ someObject){
		return null;
	},
	/**
	 * Returns a deferred object that loads an array of JSON objects for a given collection of menu choices for the selected context menu.  Its callback will
	 * receive this array as its first argument in asynchronous mode, and the deferred's <code>start</code> function will return the array directly in synchronous mode.<br/>
	 * Each object in the array follows the &nbsp;{@link com.ibm.mashups.builder.model.ContextMenuItem ContextMenuItem} 
	 * object format plus any custom extension properties that a context menu may define.<br/>
	 * @ibm-api
	 * @param {String} contextMenuId  this is id of the context menu.  Examples: ContextMenu.page, ContextMenu.space, ContextMenu.toolbox, ContextMenu.modeSelector ContextMenu.widgetSkin, or any additional extension.  Must not be <code>null</code>.
	 * @param {String} targetResourceId this is the ID of the resource to associate the decision for this menu choice with.  This will be passed into the respective contextmenu functions and must not be <code>null</code>.  
	 * @param {String} targetResourceType this is the resource type of the resource to associate the decision for this menu choice with.  This will be passed into the respective contextmenu functions.  May be <code>null</code>.
	 * Possible values for this could be "com.ibm.mashups.enabler.space.SpaceNode", "com.ibm.mashups.iwidget.widget.IWidgetDefinition", "com.ibm.mashups.enabler.navigation.NavigationNode",
	 * or any other type used by an extension.  
	 * @param {Object} someObject this allows an arbitrary object to be passed into the context menu items' functions in case they need additional data for making the decisions.  
	 * @type {com.ibm.mashups.enabler.Deferred}
	 * @return {com.ibm.mashups.enabler.Deferred} A deferred object used to start this operation,
	 * never <code>null</code>. The return value when executed through the
	 * deferred object is A JSON object array of menu choices; may be an empty array if no choices exist for that context menu, but never <code>null</code>.
	 * The deferred supports synchronous and asynchronous operations.
	 */
	getContextMenuDeferred:function(/*String*/ contextMenuId, /*String*/ targetResourceId, /*String*/ targetResourceType, /*Object*/ someObject){
		return null;
	},
	
	/**
	 * Register a context menu loader at a particular id that will be used to load items when requested
	 * by the <code> getContextMenu</code> or <code> getContextMenuDeferred</code> functions using the registered id.  A default loader
	 * will be used if nothing has been registered with a particular contextMenuId when requested.
	 * This function replaces any existing context menu loader that was registered to the same contextMenuId.
	 * Registering a <code>null</code> context menu loader will effectively remove any existing loader that is
	 * already registered to the same contextMenuId.
	 * @ibm-spi
	 * @param {String} contextMenuId	id of menu to retrieve contributions from. Must not be <code>null</code>.
	 * @param {com.ibm.mashups.builder.model.ContextMenuLoader} contextMenuLoader		contextMenuLoader to be register to a particular id. May be <code>null</code>.
	 * @return {Boolean} boolean indicating if the registration was successful
	 */
	registerContextMenu: function(/*String*/ contextMenuId, /*com.ibm.mashups.builder.model.ContextMenuLoader*/ contextMenuLoader) {
	},
	
	/**
	 * Adds a contribution to a context menu with the given contextMenuId.  The contribution defines a 
	 * set of menu items to be inserted in a group into the given context menu.  When the context menu is
	 * requested, its base contents as well as any contributions that have been added to it will be merged
	 * such that contribution items are kept together and are ordered relative to their contribution, then
	 * merged together into a flat list of items.
	 * @ibm-spi
	 * @param {String} contextMenuId	id of menu to contribute to. Must not be <code>null</code>.
	 * @param {com.ibm.mashups.builder.model.ContextMenuLoader} contributionLoader	contribution to add to the menu.
	 * @param {Number} ordinal	Optional value used for relative ordering with the base items and other contributions.<br/>
	 * @return {String} the id of the contribution that was added, or <code>null</code> if the contribution was <code>null</code>.
	 * This id may be used to remove the contribution later via the removeContribution function.
	 */
	addContribution: function(/*String*/ contextMenuId, /*com.ibm.mashups.builder.model.ContextMenuLoader*/ contributionLoader, /*Number*/ ordinal) {
	},
	/**
	 * Removes a contribution from a context menu with the given contextMenuId by the id that uniquely
	 * identifies that contribution within the particular context menu.  The id is what the system returns
	 * when the contribution is added via the addContribution function.
	 * @ibm-spi
	 * @param {String} contextMenuId	id of menu remove the contribution from. Must not be <code>null</code>.
	 * @param {String} id id of contribution to remove from the menu. 
	 * @return {com.ibm.mashups.builder.model.ContextMenuLoader} the contribution that was removed, or <code>null</code> if no match was found
	 */
	removeContribution: function(/*String*/ contextMenuId, /*String*/ id) {
	},
	/**
	 * Gets an array of all the contributions that have been added to a context menu with the given contextMenuId.
	 * @ibm-spi
	 * @param {String} contextMenuId	id of menu to retrieve contributions from. Must not be <code>null</code>.
	 * @return {com.ibm.mashups.builder.model.ContextMenuLoader[]} an array of contributions; may be an empty array, but is never <code>null</code>.
	 */
	getContributions: function(/*String*/ contextMenuId) {
	},
	/**
	 * Register a listener function which will be fired whenever a particular context menu identified
	 * by the contextMenuId is updated.  This may occur any of the following circumstances:<br/>
	 * 1) A contribution is added<br/>
	 * 2) A contribution is removed<br/>
	 * 3) A contribution ContextMenuLoader is updated via its onChange event<br/>
	 * 4) The base context menu ContextMenuLoader is updated via its onChange event<br/>
	 * <br/>
	 * Listeners may choose to respond to this event however desired, such as by requesting the context menu again
	 * in order to receive an updated set of items.
	 * @ibm-spi
	 * @param {String} contextMenuId	id of the context menu to which to register the listener. Must not be <code>null</code>.
	 * @param {Object} scope		scope in which the function will be invoked when the event is fired. May be <code>null</code> to default
	 * to global scope.
	 * @param {Function} func	function to be called whenever the context menu is changed. May be the function reference itself,
	 * or the String name of the function on the scope object. Must not be <code>null</code>.<br/>
	 * The function signature is: <code>function(contextMenuId){}</code><br/>
	 * &nbsp;&nbsp;&nbsp;&nbsp;<code>String <b>contextMenuId</b></code> id of the context menu that was updated<br/>
	 * @return {Object} a connection handle that can be used to unregister the listener.
	 */
	registerListener: function(/*String*/contextMenuId, /*Object*/scope, /*Function*/func) {
	},
	/**
	 * Register a listener function which will be fired whenever a particular context menu identified
	 * by the contextMenuId is updated.  This may occur any of the following circumstances:<br/>
	 * 1) A contribution is added<br/>
	 * 2) A contribution is removed<br/>
	 * 3) A contribution ContextMenuLoader is updated via its onChange event<br/>
	 * 4) The base context menu ContextMenuLoader is updated via its onChange event<br/>
	 * <br/>
	 * Listeners can choose to respond to this event however desired, such as by requesting the context menu again
	 * in order to receive an updated set of items.
	 * @ibm-spi
	 * @param {String} contextMenuId	id of the context menu to which to register the listener. Must not be <code>null</code>.
	 * @param {Function} func	function to be called whenever the context menu is changed. May be the function reference itself,
	 * or the String name of the function in the global scope. Must not be <code>null</code>.<br/>
	 * The function signature is: <code>function(contextMenuId){}</code><br/>
	 * &nbsp;&nbsp;&nbsp;&nbsp;<code>String <b>contextMenuId</b></code> id of the context menu that was updated<br/>
	 * @return {Object} a connection handle that can be used to unregister the listener.
	 */
	registerListener: function(/*String*/contextMenuId, /*Function*/func) {	// JSLINT-IGNORE <interface-only API>
	},
	/**
	 * Unregisters a previously registered listener to a context menu's onChange event.
	 * @ibm-spi
	 * @param {Object} handle	connection handle that was returned by a call to <code> registerListener</code>. Must not be <code>null</code>.
	 * @type void
	 */
	unregisterListener: function(/*Object*/ handle) {
	}
});
//</INTERFACE>

}
