if(!dojo._hasResource["com.ibm.mm.builder.utils.embedUtil"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.builder.utils.embedUtil"] = true;
/**
 * This class serves as a utility class to support embedding widget or mashup page 
 * to an external web page
 * @author Marshall (shiwcsdl@cn.ibm.com)
 */
dojo.provide("com.ibm.mm.builder.utils.embedUtil");
dojo.require("dojox.validate._base");

com.ibm.mm.builder.utils.embedUtil.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger("com.ibm.mm.builder.utils.embedUtil");

/**
 * This method is for preview of open search result, like dataviewer, or feed reader widget
 * It accepts the widget parameter json object. Those parameter will be applied into the widget instance which will be shown in the preview UI.
 * @param {String} widgetUrl: url of widget definition xml file
 * @param {JSON} params: widget parameter
 * @param {Object} callback: call back function when the widget is created, should pass in the function which will lauch the preview dialog
 */
com.ibm.mm.builder.utils.embedUtil.launchFeedPreview = function(widgetUrl,params,callback){
	// First, create a widget instance
	var newWidgetId = com.ibm.mm.builder.utils.embedUtil.createDummyWidgetMarkup(widgetUrl);
	
	// Second, call live text to parse dom
	var widgetPlaceHolder = dojo.byId("Embedding_Widget_TmpDiv");
	var widgetMD = com.ibm.mm.builder.utils.embedUtil.createDummyMD(widgetUrl);
	
	com.ibm.mm.builder.utils.iwidgetUtil.loadWidgets(widgetPlaceHolder, function(){
		// Add open search code to lauch preview dlg
		var widgetInstance = com.ibm.mashups.iwidget.model.Factory.getGlobalWidgetModel().find(newWidgetId).getIWidgetInstance();
        if (widgetInstance === null) {
            com.ibm.mm.builder.utils.embedUtil.LOGGER.severe("launchFeedPreview",dojo.string.substitute(com.ibm.mm.builder.coreWidgetsMessages.E_EMBEDDING_NOWIDGET_ERROR_1,[newWidgetId]));
			return;
        }
        var instanceAttributes = widgetInstance.getAttributes();
		for (var paraName in params){
			if (params[paraName]){
				instanceAttributes.setItemValue(paraName,params[paraName],false);	
			} 			
		}	
		var genScript = com.ibm.mm.builder.utils.embedUtil.generateURL(widgetUrl,null,400,600,true,instanceAttributes);
		callback(genScript);
	});
	
	// Third, update title and icon if necessary
	com.ibm.mm.builder.utils.iwidgetUtil.saveWidgetInstanceTitleAndIcon(newWidgetId,widgetMD.widgetname, widgetMD.widgetIconSrc);
};

/**
 * Create a dummy widget instance markup
 * @param {String} widgetUrl
 * @return {String} the created widget instance id
 */
com.ibm.mm.builder.utils.embedUtil.createDummyWidgetMarkup = function(widgetUrl){
	var newWidgetId = com.ibm.mm.builder.utils.iwidgetUtil._getUid();
	var widgetMD = com.ibm.mm.builder.utils.embedUtil.createDummyMD(widgetUrl);
	var widgetcreator = new com.ibm.mm.builder.WidgetCreator(newWidgetId, widgetMD);
	var widgetSpan = widgetcreator.getWidget();
	widgetSpan.removeAttribute("skin");
	var widgetStr = com.ibm.mm.builder.utils.htmlUtil.getElementString(widgetSpan);
	var widgetPlaceHolder = dojo.byId("Embedding_Widget_TmpDiv");
	if (widgetPlaceHolder){
		widgetPlaceHolder.innerHTML = ""; 
	}else{
		widgetPlaceHolder = document.createElement("div");
		widgetPlaceHolder.id = "Embedding_Widget_TmpDiv";
		dojo.body().appendChild(widgetPlaceHolder);
		//widgetPlaceHolder.style.visibility = "hidden";
		widgetPlaceHolder.style.display = "none";
	}
	widgetPlaceHolder.innerHTML = widgetStr;
	return newWidgetId;
};

/**
 * create a dummy widget meta data object, in order to call widget creator to create widget instance
 * @param {String} widgetUrl
 * @return {Object} widget meta data object
 */
com.ibm.mm.builder.utils.embedUtil.createDummyMD = function(widgetUrl){
	var widgetMD = {};
	widgetMD.id = "";
	widgetMD.widgetname = "";
	widgetMD.widgetDefXML = widgetUrl;
	widgetMD.widgetIconSrc = "";
	widgetMD.widgetParam = null;
	widgetMD.onNewWire = null;
	widgetMD.systemWidget = false;
	return widgetMD;
};

/**
 * Create a widget instance markup
 * @param {String} widgetDefIdOrUrl
 * @return {String} the created widget instance id
 */
com.ibm.mm.builder.utils.embedUtil.createEmbeddingWidgetMarkup = function(widgetDefId){
	var newWidgetId = com.ibm.mm.builder.utils.iwidgetUtil._getUid();
	var widgetMD = com.ibm.mm.builder.utils.toolboxUtil.getWidgetDefinition(widgetDefId);
	var widgetcreator = new com.ibm.mm.builder.WidgetCreator(newWidgetId, widgetMD);
	var widgetSpan = widgetcreator.getWidget();
	// for embedding widget, no skin is needed
	widgetSpan.removeAttribute("skin");
	var widgetPlaceHolder = dojo.byId("Embedding_Widget_TmpDiv");
	if (widgetPlaceHolder){
		widgetPlaceHolder.innerHTML = ""; 
	}else{
		widgetPlaceHolder = document.createElement("div");
		widgetPlaceHolder.id = "Embedding_Widget_TmpDiv";
		//widgetPlaceHolder.style.visibility = "hidden";
		widgetPlaceHolder.style.display = "none";
		dojo.body().appendChild(widgetPlaceHolder);
	}
	widgetPlaceHolder.appendChild(widgetSpan);
	return newWidgetId;
};

com.ibm.mm.builder.utils.embedUtil.createEmbeddingHubWidgetMarkup = function(widgetMap){
	var newWidgetId = com.ibm.mm.builder.utils.iwidgetUtil._getUid();
	//var widgetMD = com.ibm.mm.builder.utils.toolboxUtil.getWidgetDefinition(widgetDefIdOrUrl);
	var widgetcreator = new com.ibm.mm.builder.WidgetCreator(newWidgetId, null, null, widgetMap);
	var widgetSpan = widgetcreator.getWidget();
	// for embedding widget, no skin is needed
	widgetSpan.removeAttribute("skin");
	var widgetPlaceHolder = dojo.byId("Embedding_Widget_TmpDiv");
	if (widgetPlaceHolder){
		widgetPlaceHolder.innerHTML = ""; 
	}else{
		widgetPlaceHolder = document.createElement("div");
		widgetPlaceHolder.id = "Embedding_Widget_TmpDiv";
		//widgetPlaceHolder.style.visibility = "hidden";
		widgetPlaceHolder.style.display = "none";
		dojo.body().appendChild(widgetPlaceHolder);
	}
	widgetPlaceHolder.appendChild(widgetSpan);
	return newWidgetId;
};

/**
 * Destroy embeded widget instance
 * @param {String} widgetId
 */
com.ibm.mm.builder.utils.embedUtil.destroyEmbeddedWidget = function(widgetId){
	var widgetPlaceHolder = dojo.byId("Embedding_Widget_TmpDiv");
	if (widgetPlaceHolder){
		widgetPlaceHolder.innerHTML = ""; 
	}
	var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
	eventService.publishEvent(com.ibm.mashups.iwidget.Constants.UNLOAD_WIDGETS,widgetId);
};

com.ibm.mm.builder.utils.embedUtil.createRadioButton = function(radioName,radioValue,radioOnclick,selected,title,id){
	
	var radioString = "<input type='radio' name='";
	radioString += radioName+ "' value='";
	radioString += radioValue+ "' title='";
	radioString += title+ "' id='";
	radioString += id+ "' aria-label='";
	radioString += title+ "'";
	
	
	if(radioOnclick) {
		radioString += "onclick='"+radioOnclick+"' onkeypress='(function(e,self) { if (e.charCode == dojo.keys.SPACE || e.keyCode == dojo.keys.ENTER) {self.onclick(e);} })(event,this);'";
	}
	if (selected){
		radioString += " checked>";
	}else{
		radioString += ">";
	}
	return radioString;
};

com.ibm.mm.builder.utils.embedUtil.getHeight = function(nodeId) {
	var height = dojo.byId(nodeId+"_height").value + "px";
	return height;
};	
	
com.ibm.mm.builder.utils.embedUtil.getWidth = function(nodeId) {
	var width ="auto";
	if(dojo.query("[name="+nodeId+"_emdPgWidth]")[0].checked){
		width = dojo.byId(nodeId+"_width").value + "px";
	}
	return width;
};

com.ibm.mm.builder.utils.embedUtil.createClearDiv = function(){
	var adiv = document.createElement("div");
	adiv.style.clear = "both";
	return adiv;
};

com.ibm.mm.builder.utils.embedUtil.createCheckPreview = function(uid,checked){
	var placeHolder = com.ibm.mm.builder.utils.htmlUtil.createElementWithClass("div","embedPreviewTxt","");
	placeHolder.id = uid + "_previewDiv";
	
	var placeHolderLeft = com.ibm.mm.builder.utils.htmlUtil.createElementWithClass("span","embedPreviewTxtLeft","");
	var ckBox = com.ibm.mm.builder.utils.htmlUtil.createElementWithClass("input","","");
	ckBox.id = uid + "_checkBox";
	ckBox.setAttribute("type","checkbox");
	ckBox.title = com.ibm.mm.builder.coreWidgetsStrings.I_EMBED_PREVIEW_BYDEFAULT;
	setTimeout(function(){
		ckBox.checked = checked;
	},0);
	var txtSpan = document.createElement("label");
	txtSpan.innerHTML = com.ibm.mm.builder.coreWidgetsStrings.I_EMBED_PREVIEW_BYDEFAULT;
	txtSpan.setAttribute("for",ckBox.id);
	dojo.attr(txtSpan,"widthLimit","320");
	com.ibm.mm.builder.utils.htmlUtil.truncateNodeText(txtSpan);
	
	placeHolderLeft.appendChild(ckBox);
	placeHolderLeft.appendChild(txtSpan);
	
	var placeHolderRight = com.ibm.mm.builder.utils.htmlUtil.createElementWithClass("span","embedPreviewTxtRight","");
	var closeLink = com.ibm.mm.builder.utils.htmlUtil.createElementWithClass("a","embedPgPreviewLink",com.ibm.mm.builder.coreWidgetsStrings.I_EMBED_PREVIEW_CLOSE);
	closeLink.id = uid + "_closeLink";	
	closeLink.href = 'javascript:void(0)';  // JSLINT-IGNORE:
	closeLink.title = com.ibm.mm.builder.coreWidgetsStrings.I_EMBED_PREVIEW_CLOSE;
	dojo.attr(closeLink,"widthLimit","160");
	com.ibm.mm.builder.utils.htmlUtil.truncateNodeText(closeLink);
	placeHolderRight.appendChild(closeLink);
	
	placeHolder.appendChild(placeHolderLeft);
	placeHolder.appendChild(placeHolderRight);	
	return placeHolder;
};

// validate the 	
com.ibm.mm.builder.utils.embedUtil.validateInput = function(prefixId,parent){
	// validate width
	if(dojo.query("[name="+prefixId+"_emdPgWidth]")[0].checked){
		var widthInput = dojo.byId(prefixId+"_width");
		if (widthInput){
			if (!dojox.validate.isInRange(widthInput.value,{min: 50, max: 2000})){
				alert(com.ibm.mm.builder.coreWidgetsMessages.E_EMBEDDING_WIDTHHEIGHT_INPUTERROR1_0);
				return false;
			}
			if ((parseInt(widthInput.value,10)+"")!=widthInput.value){
				alert(com.ibm.mm.builder.coreWidgetsMessages.E_EMBEDDING_WIDTHHEIGHT_INPUTERROR2_0);
				return false;				
			}
		}
	}
	
	// validate height
	var heightInput = dojo.byId(prefixId+"_height");
	if (heightInput){	
		if (!dojox.validate.isInRange(heightInput.value,{min: 50, max: 2000})){
			alert(com.ibm.mm.builder.coreWidgetsMessages.E_EMBEDDING_WIDTHHEIGHT_INPUTERROR1_0);
			return false;
		}
		if ((parseInt(heightInput.value,10)+"")!=heightInput.value){
			alert(com.ibm.mm.builder.coreWidgetsMessages.E_EMBEDDING_WIDTHHEIGHT_INPUTERROR2_0);
			return false;				
		}		
	}
	
	return true;	
};

/**
 * This method is used to call embedding generator to get the url for embedding page or embedding widget
 * @param {String} widgetXml
 * @param {String} pid
 * @param {String} width
 * @param {String} height
 * @param {Boolean} haveScroll
 * @param {JSON} itemSet
 */
com.ibm.mm.builder.utils.embedUtil.generateURL = function(widgetXml,pid,width,height,haveScroll,itemSet,sandboxed){
	var embeddingGenerator = new com.ibm.mashups.enabler.EmbeddingGenerator(widgetXml,pid);
	embeddingGenerator.setPreferences(itemSet);
	embeddingGenerator.setWidth(width);
	embeddingGenerator.setHeight(height);
	embeddingGenerator.setScrollbar(haveScroll);
 
    if (typeof sandboxed != "undefined") {
        if (sandboxed) {
            embeddingGenerator.setSandboxed(sandboxed);
        }
    }
    
	if (typeof itemSet != "undefined"){
		if (itemSet){
			embeddingGenerator.setPreferences(itemSet);
		}
	}	
	return embeddingGenerator;
};

com.ibm.mm.builder.utils.embedUtil.displayEmbedDialog = function(id, type, widgetType){	
	var widgetTypeId;
	var eventName;
	var dialogParam = {
		containerBackgroundColor: "white",
		nodeWidth:dojo.isIE?"550px":'545px',
		initLeftPosition: "100px",
		initTopPosition: "100px",
		displayCloseButton: true	
	};
	
	if(type == "page")
	{
		widgetTypeId = "embedPage";
		eventName = "com.ibm.mashups.builder.embedPage";
		
		var pageTitle = com.ibm.mm.builder.utils.pageUtil.getPageTitle(id);
		var title = dojo.string.substitute(com.ibm.mm.builder.coreWidgetsStrings.I_EMBED_PAGE_TITLE, [pageTitle]);
		
		dialogParam.id = id + "_embeddingPage";
		dialogParam.title = title;
	}
	else
	{
		widgetTypeId = "embedWidget";
		eventName = "com.ibm.mashups.builder.embedWidget";
		
		dialogParam.id = "mainEmbeddingDialog";
		dialogParam.title = dojo.string.substitute(com.ibm.mm.builder.coreWidgetsStrings.I_EMBED_WIDGET_HEADERTEXT, [widgetType]);
	}
	
	var payload = {
    	widgetTypeId: widgetTypeId,
        eventName: eventName,
        eventPayload:  id,
		requestRefresh: true,
        dialogParam:  dialogParam,
		isHubWidget: false
	};
	com.ibm.mashups.services.ServiceManager.getService("eventService").broadcastEvent("com.ibm.mashups.builder.displayInMashupDialog",payload);
};


com.ibm.mm.builder.utils.embedUtil.displayEmbedDialogForHubWidget = function(widgetMap){	
	var widgetTypeId;
	var eventName;
	var dialogParam = {
		containerBackgroundColor: "white",
		nodeWidth:dojo.isIE?"550px":'545px',
		initLeftPosition: "100px",
		initTopPosition: "100px",
		displayCloseButton: true	
	};
	
	widgetTypeId = "embedWidget";
	eventName = "com.ibm.mashups.builder.embedHubWidget";
	
	dialogParam.id = "mainEmbeddingDialog";
	dialogParam.title = dojo.string.substitute(com.ibm.mm.builder.coreWidgetsStrings.I_EMBED_WIDGET_HEADERTEXT, [widgetMap.label]);
	
	
	var payload = {
    	widgetTypeId: widgetTypeId,
        eventName: eventName,
        eventPayload:  widgetMap,
		requestRefresh: true,
        dialogParam:  dialogParam,
		isHubWidget: true
	};
	com.ibm.mashups.services.ServiceManager.getService("eventService").broadcastEvent("com.ibm.mashups.builder.displayInMashupDialog",payload);
};

}
