dojo.provide( "com.ibm.pb.data.LayoutTemplateModelStore" );

dojo.require( "com.ibm.data.CatalogMixin" );

// summary: Wrappers Enabler's Layout Template Model API into a dojo DataStore for use with dojo widgets.
// description: This data store in particular is designed to work with the Customize shelf widgets. 
dojo.declare( "com.ibm.pb.data.LayoutTemplateModelStore", [com.ibm.data.CatalogMixin], {
	constructor: function () {
		this.model = com.ibm.mashups.enabler.layouttemplate.Factory.getTemplateModel();
		this.localizedContext = com.ibm.mashups.enabler.context.Factory.getLocalizedContext();
	},
	searchable: true,
	mapItem: function(/*DatstoreItem*/item){
		// summary: Returns a map containing parameters for the given datastore item
		//		The keys must be: label, description, tags, rating, url, id, thumbnail
		//			- label, description, url, id and thumbnail are strings
		//			- tags is an array of strings
		//			- rating is a string that contains a number, note that decimal values are allowed
		//		Having all content search datastores create item maps with the same keys allows the widget to access information uniformly
		// 		If a datastore item does not have the same attributes for getValue as seen below, this function must be overridden
		//		If a datastore item does not have an appropriate parameter to  map to a key, that map entry can be omitted
		//		An example for a datastore item with a title, summary, rating and identifier:
		//			mapItem: function(/*DatastoreItem*/item){
		//				// does not include the "tags" and "url" keys
		//				var map = {};
		//				map["label"] = this.getValue(item, "title", "untitled");
		//				map["description"] = this.getValue(item, "summary", null);
		//				map["rating"] = this.getValue(item, "rating", "0.0");
		//				map["id"] = this.getValue(item, "identifier", null);
		//				map["thumbnail"] = this.getValue(item, "thumbnail", null);
		//				return map;
		//			}
		var map = {};
		map.label = "untitled3";
		var titleFar = this.getLabel(item);
		if (titleFar) {
			if ( typeof titleFar == "string" ) {
				map.label = titleFar;
			} else {
				titleFar.setFinishedCallback(function (title) {
					map.label = title ? title : "untitled2";
				});
				titleFar.start();
			}
		}
		
		map["description"] = null;

		var url = this.getValue(item, "URL", "");
		map["url"] = "dav:"+url.substring(url.indexOf("dav")+4,url.lastIndexOf("/"));  // dav url
		
		map["id"] = this.getValue(item, "ID", null);
		
		map.thumbnail = "";
		var webDavBasePath = com.ibm.mm.enabler.remote.WebDavUrlFactory.createUrl();
		var thumbnailPath = item.getMetaData( "thumbnailPath" );
		if ( thumbnailPath ) {
		   if( typeof thumbnailPath === "string" ){
			   map.thumbnail = webDavBasePath + thumbnailPath;
		   } else {
			   thumbnailPath.setFinishedCallback(function (path) {
				   map.thumbnail = webDavBasePath + path;
			   });
			   thumbnailPath.start();
		   }
		}
		
		return map;
	},
	/********************************/
	/**                            **/
	/**                            **/
	/**    dojo.data.api.Identity  **/
	/**                            **/
	/**                            **/
	/********************************/
   getIdentity: function(/* item */ item) {
		//    summary:
		//        Returns a unique identifier for an item. The return value will be
		//        either a string or something that has a toString() method.
		//    item:
		//        The item from the store from which to obtain its identifier.
		//    exceptions:
		//        Conforming implementations may throw an exception or return null if
		//        item is not an item.
		return this.getValue( "ID" );
    },
    getIdentityAttributes: function(/* item */ item) {
		//    summary:
		//        Returns an array of attribute names that are used to generate the identity.
		//        For most stores, this is a single attribute, but for some complex stores
		//        such as RDB backed stores that use compound (multi-attribute) identifiers
		//        it can be more than one. If the identity is not composed of attributes
		//        on the item, it will return null. This function is intended to identify
		//        the attributes that comprise the identity so that so that during a render
		//        of all attributes, the UI can hide the the identity information if it
		//        chooses.
		//    item:
		//        The item from the store from which to obtain the array of public attributes that
		//        compose the identifier, if any.
    	return [ "ID" ];
    },
    fetchItemByIdentity: function(/* object */ keywordArgs) {
		//    summary:
		//        Given the identity of an item, this method returns the item that has
		//        that identity through the onItem callback. Conforming implementations
		//        should return null if there is no item with the given identity.
		//        Implementations of fetchItemByIdentity() may sometimes return an item
		//        from a local cache and may sometimes fetch an item from a remote server,
		//
		//    keywordArgs:
		//        An anonymous object that defines the item to locate and callbacks to invoke when the
		//        item has been located and load has completed. The format of the object is as follows:
		//        {
		//            identity: string|object,
		//            onItem: Function,
		//            onError: Function,
		//            scope: object
		//        }
		//    The *identity* parameter.
		//        The identity parameter is the identity of the item you wish to locate and load
		//        This attribute is required. It should be a string or an object that toString()
		//        can be called on.
		//
		//    The *onItem* parameter.
		//        Function(item)
		//        The onItem parameter is the callback to invoke when the item has been loaded. It takes only one
		//        parameter, the item located, or null if none found.
		//
		//    The *onError* parameter.
		//        Function(error)
		//        The onError parameter is the callback to invoke when the item load encountered an error. It takes only one
		//        parameter, the error object
		//
		//    The *scope* parameter.
		//        If a scope object is provided, all of the callback functions (onItem,
		//        onError, etc) will be invoked in the context of the scope object.
		//        In the body of the callback function, the value of the "this"
		//        keyword will be the scope object. If no scope object is provided,
		//        the callback functions will be called in the context of dojo.global.
		//        For example, onItem.call(scope, item, request) vs.
		//        onItem.call(dojo.global, item, request)
		keywordArgs.path = keywordArgs.identity.toString();
		return this.fetch( keywordArgs );
    },
    /********************************/
	/**                            **/
	/**                            **/
	/**    dojo.data.api.Read      **/
	/**                            **/
	/**                            **/
	/********************************/
	getFeatures: function() {
		//    summary:
		//        The getFeatures() method returns an simple keyword values object
		//        that specifies what interface features the datastore implements.
		//        A simple CsvStore may be read-only, and the only feature it
		//        implements will be the 'dojo.data.api.Read' interface, so the
		//        getFeatures() method will return an object like this one:
		//        {'dojo.data.api.Read': true}.
		//        A more sophisticated datastore might implement a variety of
		//        interface features, like 'dojo.data.api.Read', 'dojo.data.api.Write',
		//        'dojo.data.api.Identity', and 'dojo.data.api.Attribution'.
		return {
			'dojo.data.api.Read': true,
			'dojo.data.api.Identity': true
		};
	},
    getValue: function( /* item */ item, /* attribute-name-string */ attribute, 
                        /* value? */ defaultValue){
		var result = defaultValue;
		var attribute_lc = attribute.toLowerCase();
	
		switch ( attribute_lc ) {
			case "title": 
				result = this.localizedContext.getTitle( item );
				break;
			case "description": 
				result = this.localizedContext.getDescription( item );
				break;
			case "id": 
				result = item.remoteFile.getName();
				break;
			default: 
				if ( this.hasAttribute( item, attribute ) ) {
					result = item[ "get" + attribute ]();
				}
		}
   		
   		return result;
    },
   getValues: function(/* item */ item, /* attribute-name-string */ attribute) {
		//    summary:
		//         This getValues() method works just like the getValue() method, but getValues()
		//        always returns an array rather than a single attribute value.  The array
		//        may be empty, may contain a single attribute value, or may contain many
		//        attribute values.
		//        If the item does not have a value for the given attribute, then getValues()
		//        will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//        returns false, then store.getValues(item, attribute) will return [].)
		//
		//    item:
		//        The item to access values on.
		//    attribute:
		//        The attribute to access represented as a string.
		//
		//    exceptions:
		//        Throws an exception if *item* is not an item, or *attribute* is not a string
		var result = [];
		var value = this.getValue( item, attribute, undefined );
		if ( value !== undefined ) {
			result.push( value );
		}
		return result;
    },
    getAttributes: function(/* item */ item) {
		//    summary:
		//        Returns an array with all the attributes that this item has.  This
		//        method will always return an array; if the item has no attributes
		//        at all, getAttributes() will return an empty array: [].
		//
		//    item:
		//        The item to access attributes on.
		//
		//    exceptions:
		//        Throws an exception if *item* is not an item, or *attribute* is not a string
		var attr = [];
		
		for (var prop in item ) {
			if (Object.prototype.hasOwnProperty.call( item, prop ) && prop.indexOf( "get" ) === 0 ) {
				attr.push( prop.substring( 3 ) );
			}
		}
		return attr;
    },
    hasAttribute: function(/* item */ item, /* attribute-name-string */ attribute) {
		//    summary:
		//        Returns true if the given *item* has a value for the given *attribute*.
		//
		//    item:
		//        The item to access attributes on.
		//    attribute:
		//        The attribute to access represented as a string.
		//
		//    exceptions:
		//        Throws an exception if *item* is not an item, or *attribute* is not a string
    	var result = false;
    	if ( item[ "get" + attribute ] !== undefined ) {
    		result = true;
    	}
    	
    	return result;
    },
    containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value) {
		//    summary:
		//        Returns true if the given *value* is one of the values that getValues()
		//        would return.
		//
		//    item:
		//        The item to access values on.
		//    attribute:
		//        The attribute to access represented as a string.
		//    value:
		//        The value to match as a value for the attribute.
		//
		//    exceptions:
		//        Throws an exception if *item* is not an item, or *attribute* is not a string
		var result = false;
		var values = this.getValues( item, attribute );

		if ( values.length > 0 ) {
			var l = values.length;
			var i = 0;
			var found = false;
			while ( !found && i < l ) {
				found = ( values[i] === value );
				i++;
			}
		}
		
		return result;
    }, 
    isItem: function(/* anything */ something) {
		//    summary:
		//        Returns true if *something* is an item and came from the store instance.
		//        Returns false if *something* is a literal, an item from another store instance,
		//        or is any object other than an item.
		//
		//    something:
		//        Can be anything.
		//
   		return something !== undefined && something.declaredClass === "com.ibm.mm.enabler.layouttemplate.LayoutTemplate";
    },
    isItemLoaded: function(/* anything */ something) {
		//    summary:
		//        Returns false if isItem(something) is false.  Returns false if
		//        if isItem(something) is true but the the item is not yet loaded
		//        in local memory (for example, if the item has not yet been read
		//        from the server).
		//
		//    something:
		//        Can be anything.
		//
		return this.isItem(something) && something.remoteFile !== undefined;
	},
	loadItem: function(/* object */ keywordArgs) {
		//    summary:
		//        Given an item, this method loads the item so that a subsequent call
		//        to store.isItemLoaded(item) will return true.  If a call to
		//        isItemLoaded() returns true before loadItem() is even called,
		//        then loadItem() need not do any work at all and will not even invoke
		//        the callback handlers.  So, before invoking this method, check that
		//        the item has not already been loaded.
		//     keywordArgs:
		//        An anonymous object that defines the item to load and callbacks to invoke when the
		//        load has completed.  The format of the object is as follows:
		//        {
		//            item: object,
		//            onItem: Function,
		//            onError: Function,
		//            scope: object
		//        }
		//    The *item* parameter.
		//        The item parameter is an object that represents the item in question that should be
		//        contained by the store.  This attribute is required.
		//    The *onItem* parameter.
		//        Function(item)
		//        The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
		//        parameter, the fully loaded item.
		//
		//    The *onError* parameter.
		//        Function(error)
		//        The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
		//        parameter, the error object
		//
		//    The *scope* parameter.
		//        If a scope object is provided, all of the callback functions (onItem,
		//        onError, etc) will be invoked in the context of the scope object.
		//        In the body of the callback function, the value of the "this"
		//        keyword will be the scope object.   If no scope object is provided,
		//        the callback functions will be called in the context of dojo.global().
		//        For example, onItem.call(scope, item, request) vs.
		//        onItem.call(dojo.global(), item, request) 
	},
    getLabel: function(/* item */ item) {
		//    summary:
		//        Method to inspect the item and return a user-readable 'label' for the item
		//        that provides a general/adequate description of what the item is.
		//
		//    description:
		//        Method to inspect the item and return a user-readable 'label' for the item
		//        that provides a general/adequate description of what the item is.  In general
		//        most labels will be a specific attribute value or collection of the attribute
		//        values that combine to label the item in some manner.  For example for an item
		//        that represents a person it may return the label as:  "firstname lastlame" where
		//        the firstname and lastname are attributes on the item.  If the store is unable
		//        to determine an adequate human readable label, it should return undefined.  Users that wish
		//        to customize how a store instance labels items should replace the getLabel() function on
		//        their instance of the store, or extend the store and replace the function in
		//        the extension class.
		//
		//    item:
		//        The item to return the label for.
		//
		//    returns:
		//        A user-readable string representing the item or undefined if no user-readable label can
		//        be generated.
		return this.localizedContext.getTitle( item );   	
    },
    getLabelAttributes: function(/* item */ item) {
		//    summary:
		//        Method to inspect the item and return an array of what attributes of the item were used
		//        to generate its label, if any.
		//
		//    description:
		//        Method to inspect the item and return an array of what attributes of the item were used
		//        to generate its label, if any.  This function is to assist UI developers in knowing what
		//        attributes can be ignored out of the attributes an item has when displaying it, in cases
		//        where the UI is using the label as an overall identifer should they wish to hide
		//        redundant information.
		//
		//    item:
		//        The item to return the list of label attributes for.
		//
		//    returns:
		//        An array of attribute names that were used to generate the label, or null if public attributes
		//        were not used to generate the label.
		return [ "Title", "Description" ];
    },
    fetch: function(/* Object */ keywordArgs) {
		//    summary:
		//        Given a query and set of defined options, such as a start and count of items to return,
		//        this method executes the query and makes the results available as data items.
		//        The format and expectations of stores is that they operate in a generally asynchronous
		//        manner, therefore callbacks are always used to return items located by the fetch parameters.
		//
		//    description:
		//        A Request object will always be returned and is returned immediately.
		//        The basic request is nothing more than the keyword args passed to fetch and
		//        an additional function attached, abort().  The returned request object may then be used
		//        to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//        fetch parameters and are not present on the 'request' object.
		//
		//        This does not mean that custom stores can not add methods and properties to the request object
		//        returned, only that the API does not require it.  For more info about the Request API,
		//        see dojo.data.api.Request
		//
		//    keywordArgs:
		//        The keywordArgs parameter may either be an instance of
		//        conforming to dojo.data.api.Request or may be a simple anonymous object
		//        that may contain any of the following:
		//        {
		//            query: query-string or query-object,
		//            queryOptions: object,
		//            onBegin: Function,
		//            onItem: Function,
		//            onComplete: Function,
		//            onError: Function,
		//            scope: object,
		//            start: int
		//            count: int
		//            sort: array
		//        }
		//        All implementations should accept keywordArgs objects with any of
		//        the 9 standard properties: query, onBegin, onItem, onComplete, onError
		//        scope, sort, start, and count.  Some implementations may accept additional
		//        properties in the keywordArgs object as valid parameters, such as
		//        {includeOutliers:true}.
		//
		//        The *query* parameter.
		//        The query may be optional in some data store implementations.
		//        The dojo.data.api.Read API does not specify the syntax or semantics
		//        of the query itself -- each different data store implementation
		//        may have its own notion of what a query should look like.
		//        However, as of dojo 0.9, 1.0, and 1.1, all the provided datastores in dojo.data
		//        and dojox.data support an object structure query, where the object is a set of
		//        name/value parameters such as { attrFoo: valueBar, attrFoo1: valueBar1}.  Most of the
		//        dijit widgets, such as ComboBox assume this to be the case when working with a datastore
		//        when they dynamically update the query.  Therefore, for maximum compatibility with dijit
		//        widgets the recommended query parameter is a key/value object.  That does not mean that th
		//        the datastore may not take alternative query forms, such as a simple string, a Date, a number,
		//        or a mix of such.  Ultimately, The dojo.data.api.Read API is agnostic about what the query
		//        format.
		//        Further note:  In general for query objects that accept strings as attribute
		//        value matches, the store should also support basic filtering capability, such as *
		//        (match any character) and ? (match single character).  An example query that is a query object
		//        would be like: { attrFoo: "value*"}.  Which generally means match all items where they have
		//        an attribute named attrFoo, with a value that starts with 'value'.
		//
		//    The *queryOptions* parameter
		//        The queryOptions parameter is an optional parameter used to specify options that may modify
		//        the query in some fashion, such as doing a case insensitive search, or doing a deep search
		//        where all items in a hierarchical representation of data are scanned instead of just the root
		//        items.  It currently defines two options that all datastores should attempt to honor if possible:
		//        {
		//            ignoreCase: boolean, //Whether or not the query should match case sensitively or not.  Default behaviour is false.
		//            deep: boolean     //Whether or not a fetch should do a deep search of items and all child
		//                            //items instead of just root-level items in a datastore.  Default is false.
		//        }
		//
		//    The *onBegin* parameter.
		//        function(size, request);
		//        If an onBegin callback function is provided, the callback function
		//        will be called just once, before the first onItem callback is called.
		//        The onBegin callback function will be passed two arguments, the
		//        the total number of items identified and the Request object.  If the total number is
		//        unknown, then size will be -1.  Note that size is not necessarily the size of the
		//        collection of items returned from the query, as the request may have specified to return only a
		//        subset of the total set of items through the use of the start and count parameters.
		//
		//    The *onItem* parameter.
		//        function(item, request);
		//        If an onItem callback function is provided, the callback function
		//        will be called as each item in the result is received. The callback
		//        function will be passed two arguments: the item itself, and the
		//        Request object.
		//
		//    The *onComplete* parameter.
		//        function(items, request);
		//
		//        If an onComplete callback function is provided, the callback function
		//        will be called just once, after the last onItem callback is called.
		//        Note that if the onItem callback is not present, then onComplete will be passed
		//        an array containing all items which matched the query and the request object.
		//        If the onItem callback is present, then onComplete is called as:
		//        onComplete(null, request).
		//
		//    The *onError* parameter.
		//        function(errorData, request);
		//        If an onError callback function is provided, the callback function
		//        will be called if there is any sort of error while attempting to
		//        execute the query.
		//        The onError callback function will be passed two arguments:
		//        an Error object and the Request object.
		//
		//    The *scope* parameter.
		//        If a scope object is provided, all of the callback functions (onItem,
		//        onComplete, onError, etc) will be invoked in the context of the scope
		//        object.  In the body of the callback function, the value of the "this"
		//        keyword will be the scope object.   If no scope object is provided,
		//        the callback functions will be called in the context of dojo.global().
		//        For example, onItem.call(scope, item, request) vs.
		//        onItem.call(dojo.global(), item, request)
		//
		//    The *start* parameter.
		//        If a start parameter is specified, this is a indication to the datastore to
		//        only start returning items once the start number of items have been located and
		//        skipped.  When this parameter is paired with 'count', the store should be able
		//        to page across queries with millions of hits by only returning subsets of the
		//        hits for each query
		//
		//    The *count* parameter.
		//        If a count parameter is specified, this is a indication to the datastore to
		//        only return up to that many items.  This allows a fetch call that may have
		//        millions of item matches to be paired down to something reasonable.
		//
		//    The *sort* parameter.
		//        If a sort parameter is specified, this is a indication to the datastore to
		//        sort the items in some manner before returning the items.  The array is an array of
		//        javascript objects that must conform to the following format to be applied to the
		//        fetching of items:
		//        {
		//            attribute: attribute || attribute-name-string,
		//            descending: true|false;   // Optional.  Default is false.
		//        }
		//        Note that when comparing attributes, if an item contains no value for the attribute
		//        (undefined), then it the default ascending sort logic should push it to the bottom
		//        of the list.  In the descending order case, it such items should appear at the top of the list.
		//
		//    returns:
		//        The fetch() method will return a javascript object conforming to the API
		//        defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//        object returned with the required functions in Request.js attached.
		//        Its general purpose is to provide a convenient way for a caller to abort an
		//        ongoing fetch.
		//
		//        The Request object may also have additional properties when it is returned
		//        such as request.store property, which is a pointer to the datastore object that
		//        fetch() is a method of.
		//
		//    exceptions:
		//        Throws an exception if the query is not valid, or if the query
		//        is required but was not supplied.
		if ( keywordArgs === undefined || keywordArgs === null ) {
			keywordArgs = {};
		}
		
		var query = keywordArgs.query;
		var keywordSearch = query ? query.keywords : undefined;
		var onBegin = keywordArgs.onBegin;
		var onItem = keywordArgs.onItem;
		var onComplete = keywordArgs.onComplete;
		var onError = keywordArgs.onError;
		var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
		var start = keywordArgs.start ? keywordArgs.start : 0;
		var count = keywordArgs.count ? keywordArgs.count : Number.POSITIVE_INFINITY;
		var sort = keywordArgs.sort ? keywordArgs.sort : undefined;
		var caseInsensitive = keywordArgs.queryOptions ? keywordArgs.queryOptions.ignoreCase : undefined;
		var sortAttribute;
		var isSortDescending;
		
		if ( sort ) {
			sortAttribute = sort.attribute;
			isSortDescending = sort.descending;
		} else {
			sortAttribute = "id";
		}
		
		var request = keywordArgs;
		request.abort = function () {
			//any way to abort an Enabler Deferred?
			this.aborted = true;
		};
		
		//These odd looking function definitions simply ensure that all of the possible events are defined to a valid
		//function - even if it's an empty function. This simplifies the iteration code a bit, saving a few conditional
		//checks later on.
		var makeOnBeginCall = onBegin ? function (size) {
			if ( !request.aborted ) {
				onBegin.call( scope, size, request );
			}
		} : function () {};
		
		//Determines whether or not an item should be accepted into the result set.
		var me = this;
		var itemAcceptFunction = function ( item ) {
			var shouldCall = !request.aborted;
			if ( keywordSearch && keywordSearch.length > 0) {
				var id = me.mapItem(item).label;
				if ( caseInsensitive ) {
					id = id.toUpperCase();
				}
				
				var len = keywordSearch.length;
				var keyword;
				for ( var i = 0; i < len; i++ ) {
					keyword = keywordSearch[i];
					
					if ( caseInsensitive ) {
						keyword = keyword.toUpperCase();
					}
					if ( id.indexOf( keyword ) < 0 ) {
						return false;
					}
				}	
			}
			return shouldCall;
		};
		
		var makeOnItemCall = onItem ? function ( item ) {
			onItem.call( scope, item, request );
		} : function () {};
		
		var me = this;
		var makeOnCompleteCall = onComplete ? function ( items ) {
			if ( !request.aborted ) {
				if ( onItem ) {
					onComplete.call( scope, null, request );
				} else {
					//Sorts based on the given attribute. The given attribute must be one of the attributes provided by
					//this.mapItem().
					if ( sortAttribute ) {
						var lc_sortAttribute = sortAttribute.toLowerCase();
						//Control sort order based on the provided options.
						var lessThan = isSortDescending ? 1 : -1;
						var greaterThan = isSortDescending ? -1 : 1;
						var equal = 0;
						
						var sortedItems = items.sort(function (a,b) {
							var retVal = undefined;
							var item1 = me.mapItem(a);
							var item2 = me.mapItem(b);
							
							var value1 = item1[ lc_sortAttribute ];
							var value2 = item2[ lc_sortAttribute ];
							
							//Sorting logic:
							//According to the dojo data API definition, any undefined values go to the end for 
							//ascending sort order and the beginning for descending sort order. We return a -1 if value1
							//is less than value2, a 0 if value1 is equal to value2, or a +1 if value1 is greater than 
							//value2. The return value is reversed (via the local variables declared outside of this
							//function) if the sort order is descending (ascending is the default).
							
							if ( !value1 && value2 ) {
								retVal = greaterThan;
							} else if ( !value1 && !value2 ) {
								retVal = equal;
							} else if ( value1 && !value2 ) {
								retVal = lessThan;
							} else if ( value1 === value2 ) {
								retVal = equal;
							} else if ( value1 > value2 ) {
								retVal = greaterThan;
							} else if ( value1 < value2 ) {
								retVal = lessThan;
							} else {
								retVal = equal;
							}
							
							return retVal;
						});
						
						//Make sure we didn't have a failure or lose an item along the way - probably better to return
						//out of order items instead of nothing or an incomplete result set.
						if ( sortedItems !== null && sortedItems.length === items.length ) {
							items = sortedItems.splice( start, count );
						}
					}	
					
					onComplete.call( scope, items, request );
				}
			}
		} : function () {};
		
		var makeOnBeginCall = onBegin ? function ( size ) {
			if ( !request.aborted ) {
				onBegin.call(scope, size, request );
			}
		} : function () {};
		
		var path = query ? query.path : query;
		this._loadItems( path, start, count, itemAcceptFunction, makeOnBeginCall, makeOnItemCall, makeOnCompleteCall );
		
		return request;		
	},
	_loadItems: function ( path, start, count, itemAcceptFunction, makeOnBeginCall, makeOnItemCall, makeOnCompleteCall ) {
		//No specific path is specified or all items are requested. 
		if ( path === undefined || path === null || path === "*" ) {
			var me = this;
			var it = this.model.iterator();
			//The iterator has to load before it can be used. Can switch to setForEachCallback once that is fully implemented.
			it.setFinishedCallback(function () {
				var size = it.size();
				size.setFinishedCallback( function ( totalCount ) {
					var results = [];
					var pending = 0;
					returnedCount = 0;
					var delayedStarts = [];
					var threshold = 2;
					while ( it.hasNext() ) {
						var item = it.next();
						item.setFinishedCallback(function (object) { 
							if ( object && itemAcceptFunction( object ) ) {
								results.push( object );
								//Before an item is fully "loaded", we need to ensure that all remote artifacts are
								//loaded since the widgets make no allowance for a title or description being an 
								//asynchronous call away. Requesting the title here forces the item to load the remote
								//properties file for the current locale that defines the title and description.
								var locale = me.localizedContext.getLocale( object );
								//The available locales are local to the item once the initial PROPFIND occurs. If a 
								//title exists for the current configured locale, then this item isn't finished until 
								//that remote properties file is loaded. Otherwise, just be done.
								if ( locale ) {
									var d = object.getTitle( locale );
									d.setFinishedCallback(function () {
										var d2 = object.getMetaData( "" );
										d2.setFinishedCallback(function () {
											pending--;
											makeOnItemCall( object );
											returnedCount++;
											if ( pending < threshold && delayedStarts.length > 0 ) {
												pending++;
												delayedStarts.pop().start();
											}
											if ( returnedCount == totalCount ) {
												// make callbacks
												makeOnBeginCall( results.length );
												makeOnCompleteCall( results );
											}
										});
										d2.start();
									});
									d.start();
								} else {
									pending--;
									makeOnItemCall( object );
								}
							} else {
								returnedCount++;
								pending--;
								if ( pending < threshold && delayedStarts.length > 0 ) {
									pending++;
									delayedStarts.pop().start();
								}
								if ( returnedCount == totalCount ) {
									// make callbacks
									makeOnBeginCall( results.length );
									makeOnCompleteCall( results );
								}
							}	 
						});
						if ( pending >= threshold ) {
							delayedStarts.push( item );
						} else {
							pending++;
							item.start();
						}
					}
				});
				size.start();
			});
			it.start();
		} else {
			var deferred = this.model.find( path );
			deferred.setFinishedCallback(function (object) {
				makeOnBeginCall( 1 );
				makeOnItemCall( object );
				makeOnCompleteCall( object );
			});
			deferred.start();
		}
	}
    /********************************/
	/**                            **/
	/**                            **/
	/**    dojo.data.api.Write     **/
	/**                            **/
	/**                            **/
	/********************************/
	// deleteItem: function(/* item */ item){
// 		//	summary:
// 		//		Deletes an item from the store.
// 		//
// 		//	item: 
// 		//		The item to delete.
// 		//
// 		//	exceptions:
// 		//		Throws an exception if the argument *item* is not an item 
// 		//		(if store.isItem(item) returns false).
// 		//	example:
// 		//	|	var success = store.deleteItem(kermit);
// 		var success = false;
// 		
// 		if ( this.isItem( item ) ) {
// 			this.model.remove( item );
// 			success = true;
// 		}
// 		
// 		return success;
// 	},
// 	setValue: function(	/* item */ item, 
// 						/* string */ attribute,
// 						/* almost anything */ value){
// 		//	summary:
// 		//		Sets the value of an attribute on an item.
// 		//		Replaces any previous value or values.
// 		//
// 		//	item:
// 		//		The item to modify.
// 		//	attribute:
// 		//		The attribute of the item to change represented as a string name.
// 		//	value:
// 		//		The value to assign to the item.
// 		var attribute = attribute.toLowerCase();
// 		var success = false;
// 		switch ( attribute ) {
// 			case "title": 
// 				item.setTitle( value.locale, value.string );
// 				success = true;
// 			break;
// 			case "description": 
// 				item.setDescription( value.locale, value.string );
// 				success = true;
// 			break;
// 			case "metadata": 
// 				item.setMetaData( value.name, value.value );
// 				success = true;
// 			break;
// 		}
// 		
// 		return success;
// 	},
// 	setValues: function(/* item */ item, /* string */ attribute, /* array */ values) {
// 		//    summary:
// 		//        Adds each value in the *values* array as a value of the given
// 		//        attribute on the given item.
// 		//        Replaces any previous value or values.
// 		//        Calling store.setValues(x, y, []) (with *values* as an empty array) has
// 		//        the same effect as calling store.unsetAttribute(x, y).
// 		//
// 		//    item:
// 		//        The item to modify.
// 		//    attribute:
// 		//        The attribute of the item to change represented as a string name.
// 		//    values:
// 		//        An array of values to assign to the attribute..
// 		//
// 		//    exceptions:
// 		//        Throws an exception if *values* is not an array, if *item* is not an
// 		//        item, or if *attribute* is neither an attribute object or a string.
// 		//    examples:
// 		//        var success = store.setValues(kermit, "color", ["green", "aqua"]);
// 		//        success = store.setValues(kermit, "color", []);
// 		//        if (success) {assert(!store.hasAttribute(kermit, "color"));}
// 		var l = values.length;
// 		var success = true;
// 		for ( var i = 0; i < l; i++ ) {
// 			success = success && this.setValue( item, attribute, values[i] );
// 		}
// 		
// 		return success;
//     },
// 	newItem: function(/* Object */ keywordArgs, /*Object?*/ parentInfo) {
// 		//   summary:
// 		//        Returns a newly created item.  Sets the attributes of the new
// 		//        item based on the *keywordArgs* provided.  In general, the attribute
// 		//        names in the keywords become the attributes in the new item and as for
// 		//        the attribute values in keywordArgs, they become the values of the attributes
// 		//        in the new item.  In addition, for stores that support hierarchical item
// 		//        creation, an optional second parameter is accepted that defines what item is the parent
// 		//        of the new item and what attribute of that item should the new item be assigned to.
// 		//        In general, this will assume that the attribute targetted is multi-valued and a new item
// 		//        is appended onto the list of values for that attribute.
// 		//
// 		//    keywordArgs:
// 		//        A javascript object defining the initial content of the item as a set of JavaScript 'property name: value' pairs.
// 		//    parentInfo:
// 		//        An optional javascript object defining what item is the parent of this item (in a hierarchical store.  Not all stores do hierarchical items),
// 		//        and what attribute of that parent to assign the new item to.  If this is present, and the attribute specified
// 		//        is a multi-valued attribute, it will append this item into the array of values for that attribute.  The structure
// 		//        of the object is as follows:
// 		//        {
// 		//            parent: someItem,
// 		//            attribute: "attribute-name-string"
// 		//        }
// 		//
// 		//    exceptions:
// 		//        Throws an exception if *keywordArgs* is a string or a number or
// 		//        anything other than a simple anonymous object.
// 		//        Throws an exception if the item in parentInfo is not an item from the store
// 		//        or if the attribute isn't an attribute name string.
// 		//    examples:
// 		//        var kermit = store.newItem({name: "Kermit", color:[blue, green]});
// 		var path = keywordArgs.path;
// 		var metadata = keywordArgs.metadata;
// 		var titles = keywordArgs.titles;
// 		var descriptions = keywordArgs.descriptions;
// 		
// 		var item = this.model.create({ name: path });
// 		if ( metadata ) {
// 			for (var prop in metadata ) {
// 				if (Object.prototype.hasOwnProperty.call( metadata, prop ) ) {
// 					item.setMetaData( prop, metadata[prop] );
// 				}
// 			}
// 		}
// 		if ( titles ) {
// 			for (var locale in titles ) {
// 				if (Object.prototype.hasOwnProperty.call( titles, locale ) ) {
// 					item.setMetaData( locale, titles[locale] );
// 				}
// 			}
// 		}
// 		if ( descriptions ) {
// 			for (var locale in descriptions ) {
// 				if (Object.prototype.hasOwnProperty.call( descriptions, locale ) ) {
// 					item.setMetaData( locale, descriptions[locale] );
// 				}
// 			}
// 		}
// 		return item;
// 	},
// 	save: function(/* object */ keywordArgs) {
// 		//    summary:
// 		//        Saves to the server all the changes that have been made locally.
// 		//        The save operation may take some time and is generally performed
// 		//        in an asynchronous fashion.  The outcome of the save action is
// 		//        is passed into the set of supported callbacks for the save.
// 		//
// 		//    keywordArgs:
// 		//        {
// 		//            onComplete: function
// 		//            onError: function
// 		//            scope: object
// 		//        }
// 		//
// 		//    The *onComplete* parameter.
// 		//        function();
// 		//
// 		//        If an onComplete callback function is provided, the callback function
// 		//        will be called just once, after the save has completed.  No parameters
// 		//        are generally passed to the onComplete.
// 		//
// 		//    The *onError* parameter.
// 		//        function(errorData);
// 		//
// 		//        If an onError callback function is provided, the callback function
// 		//        will be called if there is any sort of error while attempting to
// 		//        execute the save.  The onError function will be based one parameter, the
// 		//        error.
// 		//
// 		//    The *scope* parameter.
// 		//        If a scope object is provided, all of the callback function (
// 		//        onComplete, onError, etc) will be invoked in the context of the scope
// 		//        object.  In the body of the callback function, the value of the "this"
// 		//        keyword will be the scope object.   If no scope object is provided,
// 		//        the callback functions will be called in the context of dojo.global.
// 		//        For example, onComplete.call(scope) vs.
// 		//        onComplete.call(dojo.global)
// 		//
// 		//    returns:
// 		//        Nothing.  Since the saves are generally asynchronous, there is
// 		//        no need to return anything.  All results are passed via callbacks.
// 		//    examples:
// 		//        store.save({onComplete: onSave});
// 		//        store.save({scope: fooObj, onComplete: onSave, onError: saveFailed});
// 		var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
// 		var d = this.model.commit();
// 		d.setFinishedCallback(function () {
// 			onComplete.call( scope );
// 		});
// 	},
// 	revert: function() {
// 		//    summary:
// 		//        Discards any unsaved changes.
// 		//    description:
// 		//        Discards any unsaved changes.
// 		//
// 		//    examples:
// 		//        var success = store.revert();
// 		this.model._revert();
// 		return true;
// 	},
// 	isDirty: function(/* item? */ item) {
// 		//    summary:
// 		//        Given an item, isDirty() returns true if the item has been modified
// 		//        since the last save().  If isDirty() is called with no *item* argument,
// 		//        then this method returns true if any item has been modified since
// 		//        the last save().
// 		//
// 		//    item:
// 		//        The item to check.
// 		//
// 		//    exceptions:
// 		//        Throws an exception if isDirty() is passed an argument and the
// 		//        argument is not an item.
// 		//    examples:
// 		//        var trueOrFalse = store.isDirty(kermit); // true if kermit is dirty
// 		//        var trueOrFalse = store.isDirty();       // true if any item is dirty
// 		var result = false;
// 		
// 		if ( item ) {
// 			result = item._isDirty();
// 		} else {
// 			result = this.model.isDirty();
// 		}
// 		
// 		return result;
// 	}
});
			 
