/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.require("com.ibm.mm.builder.iwidget.wiringscreen.AutoWiringNotification");
dojo.require("com.ibm.mm.builder.iwidget.wiringscreen.AutoWiringDisabledTip");
dojo.require("com.ibm.mm.builder.iwidget.wiringscreen.WiringCanvas");
dojo.require("com.ibm.mm.builder.utils.userUtil");

dojo.declare("com.ibm.mm.builder.systemwidget.autoWiringManager", null, {
	
	disabledTip: null,
	
	hwcBtnPos:null,
	
	displaySuccessNotification: false,
	
	autoWireWidget: null,
	
	layoutParser: com.ibm.pb.layout.LayoutParser,
	
    onLoad: function(){
		var widgetCtx=this.iContext;
		this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger("com.ibm.mm.builder.systemwidget.autoWiringManager");
    },
	
	onWireInterfaceDisplay:function(iEvent){
    	var widgetId = iEvent.payload;
		if (!widgetId) {
			return;
		}
		
		var currentPage = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage();
		var isAWEnabled = currentPage.getAutoWiringEnablement();
		if (isAWEnabled) {
			return;
		}
	},
	
	displayMessage: function(type,title,message) {
		var statusMessage = new com.ibm.widgets.StatusMessage(type, title, message);
		dojo.publish("/message/status",[{message: statusMessage, uid: "ibmStatusBox"}]);
	},
	
	isWidgetHidden: function(lcId) {
		var lcNode = this.layoutParser.findById(lcId);
		if(dojo.attr(lcNode.parentNode, "name") == "ibmHiddenWidgets")
			return true;
		else return false;
	},
	
	onAddNewWire: function(iEvent) {
		var wireInfo = iEvent.payload;
		var added = com.ibm.mm.builder.utils.wireUtil.addWireCSA2(wireInfo.sourceWidgetId, 
			wireInfo.sourceEvent, wireInfo.targetWidgetId, wireInfo.targetEvent);
		if(added) {	// hide the source widget
			if(!this.isWidgetHidden(wireInfo.sourceWidgetId)) {
				com.ibm.pb.contextMenu.sharedActions.hideControl(wireInfo.sourceWidgetId);
			}
			// quick wire created
			var title = com.ibm.mm.builder.coreWidgetsStrings.I_QUICK_WIRE_CREATED_TITLE;
			var msg = dojo.string.substitute(
					com.ibm.mm.builder.coreWidgetsStrings.I_QUICK_WIRE_CREATED, 
					[com.ibm.mm.builder.utils.iwidgetUtil.getWidgetTitleCSA2(wireInfo.sourceWidgetId),
					wireInfo.sourceEvent,
					com.ibm.mm.builder.utils.iwidgetUtil.getWidgetTitleCSA2(wireInfo.targetWidgetId)]);
			this.displayMessage("info", title, msg);
		} else {
			this._displayWirePane(wireInfo.sourceWidgetId);
			// quick wire already created
			var title = com.ibm.mm.builder.coreWidgetsStrings.I_QUICK_WIRE_ALREADY_CREATED_TITLE;
			var msg = dojo.string.substitute(
					com.ibm.mm.builder.coreWidgetsStrings.I_QUICK_WIRE_ALREADY_CREATED, 
					[com.ibm.mm.builder.utils.iwidgetUtil.getWidgetTitleCSA2(wireInfo.sourceWidgetId),
					com.ibm.mm.builder.utils.iwidgetUtil.getWidgetTitleCSA2(wireInfo.targetWidgetId)]);
			this.displayMessage("info", title, msg);
		}
	},
	
	/**
	 * handle:
	 * com.ibm.mashups.services.ServiceManager.getService("eventService")
	 * 	.broadcastEvent("com.ibm.mashups.builder.createAutoWire", <lcID>);
	 * @param {Object} iEvent
	 */
	autoWiringWidget:function(iEvent){
		var widgetId = iEvent.payload;
		if (!widgetId) {
			return;
		}
		
		var currentPage = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage();
		var isAWEnabled = currentPage.getAutoWiringEnablement();
		if(isAWEnabled) {
			// the auto wire enablement of the widget is checked when trigger this
			var autoWiringResult = com.ibm.mm.builder.utils.wireUtil.setupAutoWiring(widgetId);
			this.initAutoWiringSettings();
			if(autoWiringResult.hasAutoWire && this.displaySuccessNotification) {
				// delay display noti after the widget DIV is ready
				this.autoWireWidget = {id: widgetId, result: autoWiringResult};
				this._displayNotiWhenWidgetReady(widgetId);
			}
		}
	},
	
	_displayNotiWhenWidgetReady: function(widgetId) {
		window.setTimeout(dojo.hitch(this, function(){
			var div = this.layoutParser.findById(widgetId);
			if(div) {
				this.onLayoutRefreshed();
			} else {
				this._displayNotiWhenWidgetReady();
			}
		}), 100);
	},
	
	onLayoutRefreshed: function() {
		if(this.autoWireWidget) {
			var location = this.getDialogLocation(this.autoWireWidget.id);
			var notiDlg = new com.ibm.mm.builder.iwidget.wiringscreen.AutoWiringNotification();
			notiDlg.showAutoWiringResult(this.autoWireWidget.id, this.autoWireWidget.result, location);
			
			this.autoWireWidget = null;
		}
	},
	
	_displayWirePane: function(sourceId) {
		var widgetIcon = com.ibm.mm.builder.utils.iwidgetUtil.getWidgetIconCSA2(sourceId);
		var dialogParam = {
			nodeWidth: "486px",
			title: com.ibm.mm.builder.coreWidgetsStrings.I_WIRING_NAME,
			titleIconUrl: widgetIcon,
			leftBarWidth: "18px",
			rightBarWidth: "12px",
			displayCloseButton: true,
			hasRightBottomResizer: false
		};
		var payload = {
	    	widgetTypeId: "wireInterface",
	        eventName:  "com.ibm.mashups.builder.editWire",
	        eventPayload:  sourceId,
	        dialogParam:  dialogParam,
	        requestRefresh: true
		};
		com.ibm.mashups.services.ServiceManager.getService("eventService")
			.broadcastEvent("com.ibm.mashups.builder.displayInMashupDialog", payload);
	},
	
	onQuickWireCreate:function(iEvent) {
		var wireInfo = iEvent.payload;
		/*
    	 * The payload passed by should be a JSON object like
    	 * {
    	 *     sourceWidgetId: id of the widget which is dropped onto another widget
    	 *     targetWidgetId: id of the widget  which another widget is dropped on
    	 * }
    	 */
		var sourceLcId = wireInfo.sourceWidgetId;
		var targetLcId = wireInfo.targetWidgetId;
		
		var widgetUtil = com.ibm.mm.builder.utils.iwidgetUtil;
		var wireUtil =  com.ibm.mm.builder.utils.wireUtil;
		
		var sourceLC = widgetUtil.getLayoutControl(sourceLcId);
		var targetLC = widgetUtil.getLayoutControl(targetLcId);
		
		var sourceEvent = wireUtil.getWidgetPublishedEventsCSA2(sourceLC)[0];
		var targetEvent = wireUtil.getWidgetHandledEventsCSA2(targetLC)[0];
		
		var cannotMsgTitle = com.ibm.mm.builder.coreWidgetsStrings.I_QUICK_WIRE_CANNOT_CREATE_TITLE;
		var cannotMsg = dojo.string.substitute(
				com.ibm.mm.builder.coreWidgetsStrings.I_QUICK_WIRE_CANNOT_CREATE, 
				[com.ibm.mm.builder.utils.iwidgetUtil.getWidgetTitleCSA2(sourceLcId),
				com.ibm.mm.builder.utils.iwidgetUtil.getWidgetTitleCSA2(targetLcId)]);
		
		if(sourceEvent && targetEvent) {
			var compatible = wireUtil.isEventCompatibleCSA2(sourceEvent, targetEvent, 1);
			if(compatible) {
				var quickWireInfo = {
					"sourceWidgetId": sourceLcId,
					"sourceEvent": sourceEvent.getName(),
					"targetWidgetId": targetLcId,
					"targetEvent": targetEvent.getName()
				};
				com.ibm.mashups.services.ServiceManager.getService("eventService")
					.broadcastEvent("com.ibm.mashups.builder.addNewWire", quickWireInfo);
					
			} else {
				this._displayWirePane(sourceLcId);
				// cannot create quick wire
				this.displayMessage("info", cannotMsgTitle, cannotMsg);
			}
		} else {
			this._displayWirePane(sourceLcId);
			// cannot create quick wire
			this.displayMessage("info", cannotMsgTitle, cannotMsg);
		}
		
	},
	
	initAutoWiringSettings:function(){
		var user = com.ibm.mm.builder.utils.userUtil.getCurrentUser();
		var lookaside = com.ibm.mm.builder.utils.userUtil.isLookasideEnabled("com.ibm.mashups.user.AUTO_WIRE_ENABLED_NOTIFICATIONS");
		if (!lookaside) { //lookaside is disabled
			this.displaySuccessNotification = false;
		}
		else {
			var v1 = user.getAttribute("com.ibm.mashups.user.AUTO_WIRE_ENABLED_NOTIFICATIONS");
			if(v1 === true || v1 == "true"){
				this.displaySuccessNotification = true;
			}else{
				this.displaySuccessNotification = false;
			}
		}
	},
	
	onHWCPositionPublished:function(iEvent){
		this.hwcBtnPos = iEvent.payload;
	},
	
	getDialogLocation:function(widgetId){
		var div = this.layoutParser.findById(widgetId);
		var coords = dojo.coords(div);
		x = coords.x + 15;
		y = coords.y + 15;
		var location = {
			x: x, y: y, inNVA: false
		};
		return location;
	},
    
	onWireGraphDisplay:function(iEvent){
		var widgetId = iEvent.payload;
		this.displayWireGraph(widgetId,true);
	},
	
	/**
	 * Display the wire graph according to the widget id
	 */
	displayWireGraph:function(widgetId,alone){
		var existingCanvas = dijit.byId(widgetId + "_wiringCanvas");
		if (existingCanvas) {
			existingCanvas.destroy();
		}
		
		this.wiringCanvas = new com.ibm.mm.builder.iwidget.wiringscreen.WiringCanvas({
			id: widgetId + "_wiringCanvas",
			selectedWidgetId: widgetId,
			alone: alone
		});
		this.wiringCanvas.showGraph();
	},
	
	onUnload:function(){
		if (this.wiringCanvas) {
			this.wiringCanvas.destroy();
			this.wiringCanvas = null;
		}
	}
});
