/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.mm.builder.spaceCreateWidget");

dojo.require("dojo.string");
dojo.require("com.ibm.mm.builder.iwidget.spacemanager.widget.ThemesListControl");
dojo.require("com.ibm.mm.catalog.opensearch.ResultParser");
dojo.require("com.ibm.mm.builder.dialog.DialogFooter");
dojo.require("com.ibm.mm.builder.iwidget.spacemanager.SpaceManagerActions");
dojo.require("com.ibm.mm.builder.iwidget.spacemanager.SpaceManagerConstants");

dojo.declare("com.ibm.mm.builder.spaceCreateWidget", null, {

    WID: null,
    
    SPACE_CREATE_TYPE_BLANK: "blank",

    SPACE_CREATE_TYPE_TEMPLATE: "from_template",
    
    SPACE_CREATE_TYPE_SPACE: "from_space",
    
    resourceBundle: null,
    
    resourceMessage: null,
    
    canSetFavorite: true,
    
    templatesFromHub: true,
    
    eventService: com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME),
   
    LOGGER: com.ibm.mashups.enabler.logging.Logger.getLogger("com.ibm.mm.builder.spaceCreateWidget"),
	
	_templateList: null,
    
    onLoad: function() {
    	this._connections = [];
        this.widgetId = this.iContext.widgetId;
        this.WID = "_" + this.widgetId + "_";
        
        this.resourceBundle = com.ibm.mm.builder.coreWidgetsStrings;
        this.resourceMessage = com.ibm.mm.builder.coreWidgetsMessages;
        
        this.cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
        
        if (builderConfig.createSpaceOnTemplateMode) {            
            if (builderConfig.hasHub) {
                this.templatesFromHub = true;
            } else {
                this.templatesFromHub = false;
            }

        } else {
            this.templatesFromHub = false;
        }
        
        this.themesNode = dojo.byId(this.WID + "themesContainer");
        
        this.themesListControl = new com.ibm.mm.builder.iwidget.spacemanager.widget.ThemesListControl({
            widgetId: this.widgetId
        }, this.themesNode);
        dijit.setWaiState(this.themesListControl.domNode, "labelledby", this.WID + "choose_theme_label");
		
		
        this._messageClass = "com.ibm.lotus.oneui.v100.messages";
        
        dojo.registerModulePath(this._messageClass, this.iContext.io.rewriteURI("../dijit-library/oneUI.versionized/messages"));
        dojo.require(this._messageClass + ".Message");
		
		this.enablerUtil = com.ibm.mm.builder.utils.enablerUtil;
        
		this.iWidgetAttributes = this.iContext.getiWidgetAttributes();
		this.defaultTemplateENTitle = this.iWidgetAttributes.getItemValue("defaultTemplateENTitle");
    },
    
    onView: function() {
        dojo.parser.parse(this.iContext.getRootElement());
		
		this._spaceNameField = dojo.byId(this.WID + "SpaceName");
		this._connections.push(dojo.connect(this._spaceNameField, "onkeypress", this, "onPressEnter"));
        this._spaceDescField = dojo.byId(this.WID + "SpaceDesc");        
        this._blankCheck = dojo.byId(this.WID + "blank");
        this._fromTemplateCheck = dojo.byId(this.WID + "fromTemplate");
        this._templateList = dojo.byId(this.WID + "templateList");		
		this._templateList.disabled = true;
        this._spaceList = dojo.byId(this.WID + "spaceList");
		this._fromSpaceCheck = dojo.byId(this.WID + "fromSpace");
		this._makeFavoriteCheck = dojo.byId(this.WID + "makeFavorite");
		
        dojo.byId(this.WID + "space_name_label").innerHTML = this.resourceBundle.CREATE_SPACE_NAME;
        dojo.byId(this.WID + "space_desc_label").innerHTML = this.resourceBundle.CREATE_SPACE_DESC;
        dojo.byId(this.WID + "blank_label").innerHTML = this.resourceBundle.CREATE_SPACE_BLANK;
		dojo.attr(this._blankCheck,'aria-label',this.resourceBundle.CREATE_SPACE_BLANK);
        dojo.byId(this.WID + "fromTemplate_label").innerHTML = this.resourceBundle.CREATE_SPACE_FROM_TEMPLATE;
		dojo.attr(this._fromTemplateCheck,'aria-label',this.resourceBundle.CREATE_SPACE_FROM_TEMPLATE);
		
        dojo.byId(this.WID + "fromSpace_label").innerHTML = this.resourceBundle.CREATE_SPACE_FROM_SPACE;
		dojo.attr(this._fromSpaceCheck,'aria-label',this.resourceBundle.CREATE_SPACE_FROM_SPACE);
        dojo.byId(this.WID + "choose_theme_label").innerHTML = this.resourceBundle.CREATE_SPACE_CHOOSE_THEME;
        
        dojo.byId(this.WID + "makeFavorite_label").innerHTML = this.resourceBundle.CREATE_SPACE_MAKE_FAVORITE;
		dojo.attr(this._makeFavoriteCheck,'aria-label',this.resourceBundle.CREATE_SPACE_MAKE_FAVORITE);
        dojo.byId(this.WID + "limitHint").innerHTML = dojo.string.substitute(this.resourceBundle.CREATE_SPACE_FAVORITE_LIMIT, [builderConfig.favoriteSpacesMaximumNumber]);
       
	   
    },
	
    onPressEnter: function(event) {
		if(event.keyCode==13) {
 			this.save();
		}
	},

    onCreateSpace: function(iEvent) {
        this.createSpaceFromSpaceManager = iEvent.payload.createSpaceFromSpaceManager;
       	this.cleanUI();
        this.checkFavSpacesCount();
        this.loadExistingTemplates();
        this.loadExistingSpaces();
        this.loadThemes();
    },
    
    checkFavSpacesCount: function() {
    
        var spaceUtil = com.ibm.mm.builder.utils.spaceUtil;
        var spaceModel = spaceUtil.getSpaceModel();
        
        var iterator = spaceModel.findFavorites();
        iterator.start();
        var favCount = parseInt(iterator.size().start(), 10);
        
        var limit = parseInt(builderConfig.favoriteSpacesMaximumNumber, 10);
        
		var makeFavoriteCheck = this._makeFavoriteCheck;
		
        if (favCount >= limit) {
            makeFavoriteCheck.disabled = true;
            makeFavoriteCheck.checked = false;
            dojo.style(this.WID + "makeFavorite_label", "color", "#999");
            dojo.style(this.WID + "limitHint", "display", "block");
            this.canSetFavorite = false;
            
        } else {
            makeFavoriteCheck.disabled = false;
            makeFavoriteCheck.checked = true;
            dojo.style(this.WID + "makeFavorite_label", "color", "black");
            dojo.style(this.WID + "limitHint", "display", "none");
            this.canSetFavorite = true;
        }
        
        
    },
    
    _getCatalogTemplatesSearchUrl: function() {
        var url = null;
        try {
            url = com.ibm.mm.catalog.hubUtil.getHubOpenSearchUrl();
        } catch (e) {
        }
        if (url) {
			var queryKeyword = builderConfig.spaceTemplateQueryKeyword || "*",
				queryCount = builderConfig.spaceTemplateQueryCount || "50";
            return this.iContext.io.rewriteURI(url + "collection=spaces&searchTerms=" + queryKeyword + "&count=" + queryCount);
        }
        return null;
    },
    
    loadExistingTemplates: function() {
        this._templateList.options.length = 0;
        if (this.templatesFromHub) {
            this.loadTemplatesFromHub();
        } else {
            this.loadTemplatesFromEnabler();
        }
    },
    
    xmlToString: function(node) {
        if (dojo.isIE) {
			return node.xml;
		}
		else {
			var s = new XMLSerializer();
			return s.serializeToString(node);
		}
    },
    
    escapeXML: function(str) {
        if (!dojo.isString(str)) {
			return str;
		}
        return str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;").replace(/'/gm, "&#39;");
    },
    
    unescapeXML: function(str) {
        if (!dojo.isString(str)) {
			return str;
		}
        return str.replace(/&lt;/gm, "<").replace(/&gt;/gm, ">").replace(/&quot;/gm, "\\\"").replace(/&amp;/gm, "&");
        
    },
    
    loadTemplatesFromHub: function() {
        var templateList = this._templateList;
        //at least, should get one template from Hub
        var url = this._getCatalogTemplatesSearchUrl();
        dojo.xhrGet({
            url: url,
            handleAs: "text",
            preventCache: true,
            load: dojo.hitch(this, function(data) {
				var doc = dojox.data.dom.createDocument(data);
                var parser = com.ibm.mm.catalog.opensearch.ResultParserManager.mashuphub;
                var results = parser.parse(doc).getResults();
                var listTitle = [];
                for (var i = 0, n = results.length; i < n; i++) {
					var resultT = results[i];
					var templateDisplayName = "";
					if (resultT.title){
						var localizedTitles = resultT.title;
						templateDisplayName = this._getLocalizedTitle(localizedTitles);
					}else{
						templateDisplayName = resultT.name;
					}
					var op = new Option(templateDisplayName, resultT.importServiceURL);
					dojo.attr(op, "title", templateDisplayName);					
                    templateList.options.add(op);
					listTitle.push(templateDisplayName);
                }
				dojo.attr(templateList, "title", listTitle.join(" "));
            }),
            error: dojo.hitch(this, function() { })
        });
        this._connections.push(dojo.connect(templateList, "onchange", dojo.hitch(this, function() {
            dojo.attr(templateList, "title", templateList.options[templateList.selectedIndex].innerHTML);
        })));
    },
	
	_getLocalizedTitle: function(locObj){
		var templateTitle = null;
		var loc = dojo.locale;
		var currentLocale = (dojo.isString(loc) && loc.length > 0) ? loc.replace(/-/g, "_").toLowerCase() : null;
		for(var key in locObj){
			var normalizedLocKey = (dojo.isString(key) && key.length > 0) ? key.replace(/-/g, "_").toLowerCase() : null;
			if (currentLocale == normalizedLocKey){
				templateTitle = locObj[key];
				break;
			}
		}
		if (!templateTitle){
			templateTitle = locObj[ibmConfig["default.locale"]];
		}
		return templateTitle;
	},	
    
    loadTemplatesFromEnabler: function() {
    
        var templateList = this._templateList;
        
        var templateModel = com.ibm.mashups.enabler.model.Factory.getTemplateModel();
        templateModel.setStrategy(new com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy(5));
        var listTitle = [];
		
		var templateIt = templateModel.iterator();
        templateIt.setCursorPosition(0);
			
        //at least, should get one template from enabler api
        while (templateIt.hasNext()) {
            var node = templateIt.next();
			var nodeTitle = this.enablerUtil.getNodeTitle(node);
			
			var op = new Option(nodeTitle || this.resourceBundle.I_SPACE_MANAGER_NO_TITLE, node.getID());
			dojo.attr(op, "title", nodeTitle || this.resourceBundle.I_SPACE_MANAGER_NO_TITLE);
            templateList.options.add(op);
			if(this.defaultTemplateENTitle === node.getTitle("en")){
				op.selected = "selected";
			}
			listTitle.push(nodeTitle || this.resourceBundle.I_SPACE_MANAGER_NO_TITLE);
        }
        
        dojo.attr(templateList, "title", listTitle.join(" "));
    },
    
    loadExistingSpaces: function() {
    
        var spaceList = this._spaceList;
        spaceList.options.length = 0;
        dojo.attr(spaceList, "title", this.resourceBundle.CREATE_SPACE_EXISTING_SPACE);
        
        var spaceUtil = com.ibm.mm.builder.utils.spaceUtil;
        var spaceModel = spaceUtil.getSpaceModel();
        var iter = spaceModel.iterator();

		var hasSpaces = iter.hasNext();
		var listTitle = [];
		
		if(hasSpaces) {
	        while (iter.hasNext()) {
	            var node = iter.next();
		        if (com.ibm.mm.builder.commonActions.canCopySpace(node.getID())) {
				var nodeTitle = this.enablerUtil.getNodeTitle(node);
				
				var op = new Option(nodeTitle || this.resourceBundle.I_SPACE_MANAGER_NO_TITLE, node.getID());
				dojo.attr(op, "title", nodeTitle || this.resourceBundle.I_SPACE_MANAGER_NO_TITLE);
	            spaceList.options.add(op);
				
				listTitle.push(nodeTitle || this.resourceBundle.I_SPACE_MANAGER_NO_TITLE);
	        }
	        }
			dojo.attr(spaceList, "title", listTitle.join(" "));
		}
		dojo.attr(this._fromSpaceCheck,'disabled',!hasSpaces);
    },
    
    loadThemes: function() {
		
        this.themesListControl.refresh();
		 
    },
    
    cleanUI: function() {
    
        this._spaceNameField.value = "";
        this._spaceDescField.value = "";
        
        this._blankCheck.checked = true;
        this._templateList.disabled = true;
        this._spaceList.disabled = true;
        if (this.messageControl) {
            this.messageControl.hide();
        }
        
        if (dojo.isIE == 6) {
            dojo.style(this.WID + "themesContainer", "width", "300px");
        }
        
        dojo.attr(this.WID + "btn_save", "disabled", false);
        
        this._connections.push(dojo.connect(this._blankCheck, "onclick", dojo.hitch(this, function() {
            this._templateList.disabled = true;
            this._spaceList.disabled = true;
        })));
        this._connections.push(dojo.connect(this._fromTemplateCheck, "onclick", dojo.hitch(this, function() {
            this._templateList.disabled = false;
            this._spaceList.disabled = true;
        })));
        this._connections.push(dojo.connect(this._fromSpaceCheck, "onclick", dojo.hitch(this, function() {
            this._templateList.disabled = true;
            this._spaceList.disabled = false;
        })));
    },
    
    hide: function() {
		if (this.themesListControl)
			this.themesListControl.freeThemesList();
        this.eventService.broadcastEvent("com.ibm.mashups.builder.closeMashupDialog", this.iContext.widgetId);
		
    },
    
    _getTitle: function() {
        return this._spaceNameField.value;
    },
    
    _getDescription: function() {
		return this._spaceDescField.value;
    },
    
    _getSpaceCreateType: function() {
        if (this._blankCheck.checked) {
        	return this.SPACE_CREATE_TYPE_BLANK;
        } else if (this._fromTemplateCheck.checked) {
			return this.SPACE_CREATE_TYPE_TEMPLATE;
		} else {
			return this.SPACE_CREATE_TYPE_SPACE;
		}
    },
    
    _getSelectedTheme: function() {
        return this.themesListControl.selectedTheme;
    },
    
    _setTheme: function(space, theme) {
        if (!space) {
            return;
        }
        var navigationModel = com.ibm.mm.builder.utils.navigationUtil.getNavigationModel();
        
        var navRootNode = navigationModel.find(space.getRootNavigation()).start();
        navRootNode.setTheme(theme);
        
        navigationModel.commit().start();
    },
    
    _createBlankSpace: function(title, description, theme) {
        dojo.attr(this.WID + "btn_save", "disabled", true);

        var spaceUtil = com.ibm.mm.builder.utils.spaceUtil;
        var spaceModel = spaceUtil.getSpaceModel();
        
        var space = spaceModel.create();

        space.setTitle(title, builderConfig.locale);
		space.setDescription(description, builderConfig.locale);
		
		spaceUtil.setSpaceIconURL(space, this.defaultIcon());
        this._setFavorite(space);
        
        spaceModel.insert(space);
        
        var deferred = spaceModel.commit();
       
		var newNode = null;
        deferred.setOperationCallback(dojo.hitch(this, function(node, operation, status, params) {
			if(!node && status >= 400){
                this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_CREATED_FAILED_0);
                this.messageControl.show();
                dojo.attr(this.WID + "btn_save", "disabled", false);
            }else{
				newNode = node;
				if (space.isFavorite()) {
					newNode.setFavorite(true);
				}
			}
            
        }));
        
		deferred.setFinishedCallback(dojo.hitch(this, function(ret, statusCode){
			if (!newNode || statusCode >= 400) {
				this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_CREATED_FAILED_0);
                this.messageControl.show();
                dojo.attr(this.WID + "btn_save", "disabled", false);
				
            }else{
				var payload = newNode.getID();
                
                this._setTheme(newNode, theme);                
                
                if (!this.createSpaceFromSpaceManager) {
                    this.eventService.broadcastEvent("com.ibm.mashups.builder.switchSpace", payload);
                }
                
                this.messageControl.hide();
                this.hide();
				this._createFirstPage(payload);
				this.eventService.broadcastEvent("com.ibm.mashups.builder.spacemanager.spaceCreated", payload);
			}
		}));
        
		deferred.start(false);
    },
    
    _createFirstPage : function(spaceId) {
	    var spaceUtil = com.ibm.mm.builder.utils.spaceUtil;		
	    var eventService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME);
	    var navigationUtil = com.ibm.mm.builder.utils.navigationUtil;
        var spaceModel = spaceUtil.getSpaceModel();
		var space = spaceModel.find(spaceId).start();
        var parentPageId = space.getRootNavigation().getID();
        var bundle = com.ibm.mm.builder.coreWidgetsStrings;
		var payload = {
			widgetTypeId: "createPage",
			eventName: "com.ibm.mashups.builder.createPage",
			eventPayload: {"spaceId":spaceId, "parentPageId":parentPageId, "message": bundle.I_CREATE_PAGE_IN_SPACE_FIRST_PAGE_NOTE, "isFromSpaceMgr":this.createSpaceFromSpaceManager},
			requestRefresh: true,
			dialogParam: {
				nodeWidth: "675px",
				title: bundle.I_CREATE_PAGE_IN_SPACE_TITLE,
				leftBarWidth: "0px",
				rightBarWidth: "0px",
				containerBackgroundColor: "white",
				hasRightBottomResizer: false
			}
		};
		eventService.broadcastEvent("com.ibm.mashups.builder.displayInMashupDialog", payload);
    },
    
    _createSpaceFromTemplate: function(title, description, theme) {
    
        var templateList = this._templateList;
        if (templateList.options.length === 0) {
			this._setAsAriaInvalid(templateList);
            this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_NO_TEMPLATE_FOR_SPACE_CREATION_0);
            this.messageControl.show();
            return;
        }
        this._removeAriaInvalid(templateList);
        dojo.attr(this.WID + "btn_save", "disabled", true);
        if (this.templatesFromHub) {
            this._createSpaceFromHubTemplate(title, description, theme);
            
        } else {
            this._createSpaceFromEnablerTemplate(title, description, theme);
        }
        
    },
    
    _createSpaceWithTemplateID: function(title, description, theme, templateId) {
    
        var spaceUtil = com.ibm.mm.builder.utils.spaceUtil;
        var spaceModel = spaceUtil.getSpaceModel();
        
        var space = spaceModel.create({
            "templateID": templateId
        });

        space.setTitle(title, builderConfig.locale);
        space.setDescription(description, builderConfig.locale);
		
		spaceUtil.setSpaceIconURL(space, this.defaultIcon());
        this._setFavorite(space);
        
        spaceModel.insert(space);
        
        var deferred = spaceModel.commit();
       
		var newNode = null;
        deferred.setOperationCallback(dojo.hitch(this, function(node, operation, status, params) {
			if(!node && status >= 400){
                this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_CREATED_FAILED_0);
                this.messageControl.show();
                dojo.attr(this.WID + "btn_save", "disabled", false);
            }else{
				newNode = node;
				if (space.isFavorite()) {
					newNode.setFavorite(true);
				}
			}
            
        }));
        
		deferred.setFinishedCallback(dojo.hitch(this, function(ret, statusCode){
			if (!newNode || statusCode >= 400) {
				this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_CREATED_FAILED_0);
                this.messageControl.show();
                dojo.attr(this.WID + "btn_save", "disabled", false);
				
            }else{
				var payload = newNode.getID();
                
                this._setTheme(newNode, theme);
                this.eventService.broadcastEvent("com.ibm.mashups.builder.spacemanager.spaceCreated", payload);
                
                if (!this.createSpaceFromSpaceManager) {
                    this.eventService.broadcastEvent("com.ibm.mashups.builder.switchSpace", payload);
                }
                
                this.messageControl.hide();
                this.hide();
			}
		}));
        
		deferred.start(false);
    },
    
    defaultIcon: function() {
        return builderConfig.builderPath + "/images/spaceicons/icon-01.gif";
    },
    
    _createSpaceFromEnablerTemplate: function(title, description, theme) {
    
        var templateList = this._templateList;
        var templateId = templateList.options[templateList.selectedIndex].value;
        
        this._createSpaceWithTemplateID(title, description, theme, templateId);
        
    },
    
    getMashupsImportSpaceUrl: function() {
        return [location.protocol, "//", location.host, "/mum/mycontenthandler?uri=space:collection&mode=import"].join("");
    },
    
    _createSpaceFromHubTemplate: function(title, description, theme) {
    
        var templateList = this._templateList;
        
        var hubImportServiceUrl = templateList.options[templateList.selectedIndex].value;        
        hubImportServiceUrl = this.iContext.io.rewriteURI(this.unescapeXML(hubImportServiceUrl));
        
        var self = this;
        dojo.xhrPost({
        
            url: hubImportServiceUrl,
            handleAs: "json",
            
            content: {
                "authType": "Form",
                "description": description,
                "import_service_url": this.getMashupsImportSpaceUrl(),
                "title": title,
                "locale": this.cs.getValue("default.locale")
            },
            
            load: function(data) {
                var statusCode = parseInt(data.status, 10);
                
                if (statusCode >= 300 || statusCode < 200) {
                    this.error(data);
                } else {
                    var spaceId = data.spaceId.split(":")[2];
                    
                    var newSpace = com.ibm.mm.builder.utils.spaceUtil.getSpaceNode(spaceId);
                    self._setFavorite(newSpace);
                    var spaceModel = com.ibm.mm.builder.utils.spaceUtil.getSpaceModel();
                    spaceModel.commit().start();
                    
                    self._setTheme(newSpace, theme);
                    self.eventService.broadcastEvent("com.ibm.mashups.builder.spacemanager.spaceCreated", spaceId);

                    if (!self.createSpaceFromSpaceManager) {
                        self.eventService.broadcastEvent("com.ibm.mashups.builder.switchSpace", spaceId);
                    }
                    
                    self.messageControl.hide();
                    self.hide();
                }
                
            },
            
            error: dojo.hitch(this, function(data) {
                this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_CREATED_FAILED_0);
                this.messageControl.show();
                dojo.attr(this.WID + "btn_save", "disabled", false);
            })
        });
    },
    
    _setFavorite: function(space) {
    
        if (this._needSetAsFavorite(space)) {
            space.setFavorite(true);
        }
    },
    
    _needSetAsFavorite: function(space) {
        var makeFavCheck = this._makeFavoriteCheck;
        return space && !makeFavCheck.disabled && makeFavCheck.checked && this.canSetFavorite;
    },
    
    _createSpaceFromSpace: function(title, description, theme) {
    
        var spaceList = this._spaceList;
        if (spaceList.options.length === 0) {
			this._setAsAriaInvalid(spaceList);
            this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_NO_SPACE_FOR_SPACE_CREATION_0);
            this.messageControl.show();
            return;
        }
        
		this._removeAriaInvalid(spaceList);
        dojo.attr(this.WID + "btn_save", "disabled", true);
        
        var spaceUtil = com.ibm.mm.builder.utils.spaceUtil;
        var spaceModel = spaceUtil.getSpaceModel();
        
		var basedSpaceId = spaceList.options[spaceList.selectedIndex].value;
        var space = spaceModel.create({
            "clonedSpace": basedSpaceId
        });
		
        space.setTitle(title, builderConfig.locale);
        space.setDescription(description, builderConfig.locale);
        this._setFavorite(space);
        
        spaceModel.insert(space);
        var deferred = spaceModel.commit();
        
		var newNode = null;
        deferred.setOperationCallback(dojo.hitch(this, function(newSpace, operation, status, params) {
			if(!newSpace && status >= 400){
                this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_CREATED_FAILED_0);
                this.messageControl.show();
                dojo.attr(this.WID + "btn_save", "disabled", false);
            }else{
				newNode = newSpace;
				if (space.isFavorite()) {
					newNode.setFavorite(true);
				}
			}
            
        }));
        
		deferred.setFinishedCallback(dojo.hitch(this, function(ret, statusCode){
			if (!newNode || statusCode >= 400) {
				this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_CREATED_FAILED_0);
                this.messageControl.show();
                dojo.attr(this.WID + "btn_save", "disabled", false);
				
            }else{
				var payload = newNode.getID();
                this._setTheme(newNode, theme);
                
                this.eventService.broadcastEvent("com.ibm.mashups.builder.spacemanager.spaceCreated", payload);
                if (!this.createSpaceFromSpaceManager) {
                    this.eventService.broadcastEvent("com.ibm.mashups.builder.switchSpace", payload);
                }
                this.messageControl.hide();
                this.hide();
			}
		}));
		
        deferred.start(false);
    },
    
	_checkTitle: function(title) {
		title = dojo.trim(title || "");		
		
		if (!title){
			return {res:false, errorMsg: this.resourceMessage.E_SPACE_MANAGER_SPACE_NAME_IS_REQUIRED_0};
		} 		
		if (title.length > 256){
			return {res:false, errorMsg: this.resourceMessage.E_SPACE_MANAGER_SPACE_NAME_HAS_MAX_LIMIT_0};
		} 		
		var match = ibmConfig[com.ibm.mashups.enabler.services.ConfigConstants.BUILDER_TITLE_FILTER_REGEXP];		
		if (title.match(match)) {
			return {
				res: false,
				errorMsg: this.resourceMessage.E_SPACE_MANAGER_SPACE_NAME_HAS_INVALID_CHAR_0
			};
		}	
		
		return {res:true};		
	},
	
	
	_verifyTitle : function(title){
		var spaceNameInput = dojo.byId(this.WID+"SpaceName"); 
		
		var checkResult = this._checkTitle(title);       

		if (!checkResult.res) {
			this._setAsAriaInvalid(spaceNameInput);
			
			this.messageControl.attr("message", checkResult.errorMsg);			
			this.messageControl.show();			
        	return false;
		}
	    		
		this._removeAriaInvalid(spaceNameInput);
	    return true;
	},
    
    _verifyDescription: function(description) {
    
        if (!dojo.isString(description)) {
			return false;
		}
        
        if (description.length >= 512) {
			this._setAsAriaInvalid(this._spaceDescField);
            this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_DESC_HAS_MAX_LIMIT_0);
            this.messageControl.show();
            return false;
        }
        this._removeAriaInvalid(this._spaceDescField);		
        return true;
    },
    
    _verifyInput: function(title, description) {
        return this._verifyTitle(title) && this._verifyDescription(description);
    },
    
	_setAsAriaInvalid: function(domNode){
		dojo.attr(domNode, "aria-invalid", "true" );
		//this._setAsAriaInvalid(domNode, "aria-invalid", "true");
	},
	
	_removeAriaInvalid: function(domNode){
		dojo.removeAttr(domNode, "aria-invalid");
	},
	
    save: function() {
    
        var title = this._getTitle();
        var description = this._getDescription();
		if (!description) description = "";
        
        if (!this.messageControl) {
            dojo.registerModulePath("com.ibm.lotus.oneui.v100.messages", this.iContext.io.rewriteURI("../dijit-library/oneUI.versionized/messages"));
            dojo.require("com.ibm.lotus.oneui.v100.messages.Message");
            this.messageControl = new (dojo.getObject(this._messageClass).Message)({
                messageType: 'error'
            }, this.WID + "messageBox");			
            this.messageControl.hide();
        }
        
        //validation
        if (!this._verifyInput(title, description)) {
            return;
        }
        
        var theme = this._getSelectedTheme();
        var spaceType = this._getSpaceCreateType();
        
        //create space from template
        var templateList = this._templateList;
        
        if (spaceType == this.SPACE_CREATE_TYPE_BLANK) {
            this._createBlankSpace(title, description, theme);
        } else if (spaceType == this.SPACE_CREATE_TYPE_TEMPLATE) {
            this._createSpaceFromTemplate(title, description, theme);
        } else {
            this._createSpaceFromSpace(title, description, theme);
        }
    },
	
	onUnload: function() {
		dojo.forEach(this._connections, dojo.disconnect);
	}
});
