/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.iwidget.Constants");

/**
 * com.ibm.mashups.iwidget.Constants defines all the public iwidget framework constants.<br/>
 * Each constants should be referred to as : com.ibm.mashups.iwidget.Constants.WIDGET_LOADED...
 * @ibm-module iWidget2
 */

dojo.declare("com.ibm.mashups.iwidget.Constants", null, {

   /**
	*  Enabler would publish WIDGET_LOADED event when an iwidget is loaded. <br/>
  	* <b>Event</b>: com.ibm.mashups.iwidget.Constants.WIDGET_LOADED<br/>
	* <b>Value</b>:"com.ibm.mashups.iwidget.widgetloaded"<br/>
	* <b>Payload</b>: String -- widgetid<br/>
	* For example: 
	* When event is published, a name convention is use: com.ibm.mashups.iwidget.Constants.WIDGET_LOADED+"."+widgetId
	* An widget could subscribe to this event by using following SPI:
	* var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
	* eventService.subscribeEvent(com.ibm.mashups.iwidget.Constants.WIDGET_LOADED+"."+widgetId,scopeobject,"mycallbackfn");
	* @type String
	* @ibm-spi
	*/ 
	WIDGET_LOADED: "com.ibm.mashups.iwidget.widgetloaded",	
	
  /**
    *  Enabler would subscribe this event and page components could publish RESIZE_WIDGET event to indicate an iwidget need to be resized.<br/>
    *  When event is published, a name convention is used: com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET+"."+widgetId
	*  Page component should publish event by using following SPI:
	*  var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
	*  eventService.publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET+"."+widgetId,payload);
	* <b>Event</b>: com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET<br/>
	* <b>Value</b>:"com.ibm.mashups.iwidget.resizewidget"<br/>
	* <b>Payload</b>: JSON object: {"newWidth": "","newHeight": ""}<br/>
	* @type String
	* @ibm-spi
	*/
   	RESIZE_WIDGET: "com.ibm.mashups.iwidget.resizewidget",  


  /**
    *  Enabler would subscribe this event and page components could publish UNLOAD_WIDGETS event to unload certain widgets.<br/>
    *  For example: layout widget could publish this event to unload all the widgets on the page upon page switch<br/>
    *  When all the requested iwidgets are unloaded, enabler would publish event --com.ibm.mashups.iwidget.Constants.WIDGETS_UNLOADED <br/>
	* <b>Event</b>: com.ibm.mashups.iwidget.Constants.UNLOAD_WIDGETS<br/>
	* <b>Value</b>:"com.ibm.mashups.iwidget.unloadwidgets"<br/>
	* <b>Payload</b>: an array of widget id that need to be unloaded<br/>
	* @type String
	* @ibm-spi
	*/
   	UNLOAD_WIDGETS: "com.ibm.mashups.iwidget.unloadwidgets",  

  /**
    *  Enabler would publish WIDGETS_UNLOADED event when requested iwidgets are  unloaded. This event is used together with com.ibm.mashups.iwidget.Constants.UNLOAD_WIDGETS.<br/>
    * <b>Event</b>: com.ibm.mashups.iwidget.Constants.WIDGETS_UNLOADED<br/>
    * <b>Value</b>:"com.ibm.mashups.iwidget.widgetsunloaded"<br/>
    * <b>Payload</b>: an array of widget id that are unloaded<br/>
    * @type String
    * @ibm-spi
    */   
	WIDGETS_UNLOADED: "com.ibm.mashups.iwidget.widgetsunloaded", 
 /**
    *  Enabler would publish WIDGET_MODECHANGED event when requested iwidget has switched from one mode to another mode. <br/>
    * <b>Event</b>: com.ibm.mashups.iwidget.Constants.WIDGET_MODECHANGED<br/>
    * <b>Value</b>:"com.ibm.mashups.iwidget.widgetmodechanged"<br/>
    * <b>Payload</b>: JSON object: {id:"",oldMode:"",newMode:""}<br/>
    * @type String
    * @ibm-api
    */   
	WIDGET_MODECHANGED: "com.ibm.mashups.iwidget.widgetmodechanged",
	/**
	*  Enabler would subscribe this event and page components could publish CHANGE_WIDGETMODE event to change a mode of a widget.<br/>
	*  Page components should publish event as follows:	
	*  var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");
	*  eventService.publishEvent(com.ibm.mashups.iwidget.Constants.CHANGE_WIDGETMODE,payload);
	*  following api is deprecated:
	*  com.ibm.mashups.services.ServiceManager.getService("eventService").fireEvent(this.iwidgetId, "onModeChanged", {
    *        newMode: mode,
    *        rootElementId: <DOMNode>
    *    });
	*  <b>Event</b>: com.ibm.mashups.iwidget.Constants.CHANGE_WIDGETMODE<br/>
    * <b>Value</b>:"com.ibm.mashups.iwidget.changeWidgetMode"<br/>
    * <b>Payload</b>: JSON object: {id:"",newMode:"",parentNode:<DOMNode>}<br/>
    * @type String
    * @ibm-api
    */   
	CHANGE_WIDGETMODE: "com.ibm.mashups.iwidget.changewidgetmode",
	/**
    *  Enabler would subscribe WIDGET_WINDOWSTATECHANGED event when iwidget has been switched from one window state to another another. <br/>
    * <b>Event</b>: com.ibm.mashups.iwidget.Constants.WIDGET_WINDOWSTATECHANGED<br/>
    * <b>Value</b>:"com.ibm.mashups.iwidget.widgetwindowstatechanged"<br/>
    * <b>Payload</b>: JSON object: {id:"",oldWindowState:"",newWindowState:""}<br/>
    * @type String
    * @ibm-api
    */   
	WIDGET_WINDOWSTATECHANGED: "com.ibm.mashups.iwidget.widgetwindowstatechanged",
	/**
	*  Enabler would publish this event when widget is tried to change window state.<br/>
	*  <b>Event</b>: com.ibm.mashups.iwidget.Constants.CHANGE_WIDGETWINDOWSTATE<br/>
    * <b>Value</b>:"com.ibm.mashups.iwidget.changeWidgetWindowState"<br/>
    * <b>Payload</b>: JSON object: {id:"",newWindowState:"",oldWindowState:""}<br/>
    * @type String
    * @ibm-api
    */   
	CHANGE_WIDGETWINDOWSTATE: "com.ibm.mashups.iwidget.changewidgetwindowstate",
  /**
   * Event that will be published when navigation state is committed.
   * <b>Event</b>: com.ibm.mashups.iwidget.Constants.NAVSTATE_UPDATED<br/>
   * <b>Value</b>:"com.ibm.mashups.enabler.model.state.onNavStateUpdated"<br/>
   * For example: <br/>
   * An widget could subscribe to this event by using following SPI:<br/>
   * var eventService = com.ibm.mashups.services.ServiceManager.getService("eventService");<br/>
   * eventService.subscribeEvent("com.ibm.mashups.enabler.model.state.onNavStateUpdated",scope,"mycallbackfn");<br/>
   * In the callback function, an widget should use Accessor api to get the navstate they're interested </br>
   * Following example use PageAccessor as an example<br/>
   * 	var pageAccessor = com.ibm.mashups.enabler.model.state.AccessorFactory.getPageAccessor(navigationStateModel);<br/>
   *	var pageId = pageAccessor.getPageID();		<br/> 
   * @type String
   */
	NAVSTATE_UPDATED:"com.ibm.mashups.enabler.model.state.onNavStateUpdated"
 
});

com.ibm.mashups.iwidget.Constants = new com.ibm.mashups.iwidget.Constants();
