/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.mm.widget.dataViewer._WB_VER_.v210.ViewConfig");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.ColumnsList");

dojo.declare("com.ibm.mm.widget.dataViewer._WB_VER_.v210.ViewConfig", null, {
    _columns:null,
    columnManager:null,
    settings:null,
    domID:null,
    contextRoot:null,
    
    constructor:function(att,columnManager,domID,contextRoot){
        this.columnManager = columnManager;
        this.loadItemSet(att);
        this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger("com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewer");
        this.resourceBundle = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages;
        this.domID = domID;
        this.contextRoot= contextRoot;
    },
    
    loadItemSet: function(att){
        if(!att) return;
        this.settings=[];
		dojo.forEach(att.getAllNames(), function(name) {
			this.settings[name] = att.getItemValue(name) || "";
			if (this.settings[name] === "true") this.settings[name] = true;
			if (this.settings[name] === "false") this.settings[name] = false;
			if (this.settings[name] === "null") this.settings[name] = null;
		}, this);
		
		var	columns = {};
		var columnsStr = att.getItemValue("columns");
		if (columnsStr) {
			try{
				columns = dojo.fromJson(columnsStr);
				if (columns.columnList) {
					if (!columns.columnLabel) {
						columns.columnLabel = dojo.map(columns.columnList, function(item){
							return item;
						});
					}
					if (!columns.columnSorttype) {
						columns.columnSorttype = dojo.map(columns.columnList, function(item) {
							return "String";
						});
					}
				}
			} catch(e) {
				att.setItemValue("columns", "", false);
				this.LOGGER.severe("_loadItemSet", this.resourceBundle.E_ITEMSET_COLUMNS_BROKEN);
			}
		}
		this.settings.columns = columns;
		
		if (this.columnManager) {
			this.columnManager.reload(this.settings['columnHistory']);
		} else {
			this.columnManager = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.ColumnManager(this.settings['columnHistory']);
		}
      },

    getColumnSettings:function(currentSource){
        return this.columnManager.get(currentSource) || this.settings['columns'];
    },
    
    isDefaultValue:function(key,value){
        value = value+'';
        switch(key){
            case 'rowsPerView':
            case 'rowsLimit':
                return '10'==value;
            case 'tableTitleColor':
            case 'tableHeaderColor':
            case 'tableContentColor':
                return "#000000" ==value;
            case 'oddRowColor':
                return "#ffffff" ==value;
            case 'evenRowColor':
                return "#eeffff" ==value;
            case 'highLightFilterColor':
                return "#ebec78" ==value;
            case 'filterColumnOption':
                return '[]'==value;
            case 'invalidFeedMessage':
                return this.resourceBundle.E_CHECK_URL == value;
            case 'noContentMessage': 
                return this.resourceBundle.E_NO_CONTENT == value;
            case 'allContentFilteredMessage':
                return this.resourceBundle.I_ALL_CONTENT_FILTERED ==value;  
            case 'isFilter':
                return value =='true';
            case 'filterType':
                return 'filter';
            case 'isWiring':
                return value=='false';          
            case 'filterColumnType':
                return value=='text';
            case 'columnHistory':
                return '{"settingMap":{},"history":[]}';
            case 'isHighLightFilter':
            case 'horizontalView':
            case 'showHeader':
                return value =='true';
            case 'filterFunction':
                return value == 'bw';
        }
        return value == null || value == "";
    },
    
    get:function(key){
        switch(key){
            case 'rowsPerView':
            case 'rowsLimit':
                return parseInt(this.settings[key], 10);
                break;
            case 'tableTitleColor':
            case 'tableHeaderColor':
            case 'tableContentColor':
                if (!this.isColorValid(key)) return "#000000";
                break;
            case 'oddRowColor':
                if (!this.isColorValid(key)) return "#ffffff";
                break;
            case 'evenRowColor':
               if (!this.isColorValid(key)) return "#eeffff";
                break;
            case 'highLightFilterColor':
                if (!this.isColorValid(key)) return "#ebec78";
                break;
            case 'filterColumnOption':
                	try{
            			return dojo.fromJson(this.settings[key]);
            		} catch(e){
            			this.LOGGER.severe("onedit", this.resourceBundle.E_ITEMSET_FILTERCOLUMNOPTION_BROKEN);
                        return [];
            		}
                  break;
            case 'filterTargetValue':
                 return this.settings[key] || "";
                 break;
            case 'invalidFeedMessage':
                 return this.settings[key] || this.resourceBundle.E_CHECK_URL;
            case 'noContentMessage': 
                return this.settings[key] || this.resourceBundle.E_NO_CONTENT;
            case 'allContentFilteredMessage':
                return this.settings[key] || this.resourceBundle.I_ALL_CONTENT_FILTERED;
            case 'feedURL':
                return this._getFeedURL();
            case 'filterFunction':
                return this.settings[key] || 'bw';
        }
        
        return this.settings[key];
    },
    
    set:function(key,value){
        this.settings[key]=value;
    },
    
    _getFeedURL:function(){
        if(!this.settings) return null;
        if(!!this.settings["feedURL"]) return this.settings["feedURL"];
        if (!!this.settings["vURL"]) {
             return this.getUrl(dojo.fromJson(this.settings["vURL"]));
        }
    },
    
    getUrl: function(vurl) {
        var params = {};
        
        dojo.forEach(vurl.parameters, function(item) {
            params[item.paramName] =item.defaultValue;
        });
         return this.isParamEmpty(params) ? vurl.baseURL : (vurl.baseURL + "?" + dojo.objectToQuery(params));
    },
    
    isParamEmpty : function(object) {
        for (var i in object) {
            if (!dojo.isFunction(object[i])) {
                return false;
            }
        }
        return true;
    },
    
    isColorValid:function(key){
        return !!this.settings[key] && this.settings[key].indexOf('#') == 0; 
    },
    
	 resetColumnsSetting: function(data,att){
		var columnsSettings = {
			columnList: [],
			columnsAlign: {},
			columnFilter: [],
			columnsOrder: [],
			columnLabel: [],
			columnSorttype: []
		};
		var filterColumnOption = "[]";
		
		if (!data){
			viewConfig.get('columns') = columnsSettings;
			
			att.setItemValue("columns", "", false);
			att.setItemValue("filterColumnOption", filterColumnOption, false);
            //invoke in both edit/view mode,for view mode, do not invoke save,
            //in edit mode, save will be done when user click save button. 
            //att.save();
			return;
		}

		var columnList = data.columnList;

		if (this.isColumnsSettingEmptyOrStale(columnList)){
			for (var i = 0; i < columnList.length; i++) {
				columnsSettings.columnList.push(columnList[i]);
				columnsSettings.columnsAlign[columnList[i]] = "left";
				columnsSettings.columnFilter.push(true);
				columnsSettings.columnsOrder.push(i);
				columnsSettings.columnLabel.push(columnList[i]);
				columnsSettings.columnSorttype.push("String");
			}

			this.settings['columns'] = columnsSettings;
			
			
			att.setItemValue("columns", dojo.toJson(columnsSettings), false);
			//invoke in both edit/view mode,for view mode, do not invoke save,
            //in edit mode, save will be done when user click save button. 
            //att.save();
		}
		
		if (columnList && dojo.isArray(columnList)){
			filterColumnOption = dojo.toJson(columnList);
			att.setItemValue("filterColumnOption", filterColumnOption, false);
			//invoke in both edit/view mode,for view mode, do not invoke save,
            //in edit mode, save will be done when user click save button. 
            //att.save();
		}
	},
    isColumnsSettingEmptyOrStale: function(columnList){
		if (!dojo.isArray(this.get('columns')['columnList']) || !this.get('columns')['columnsAlign'] ||
			!dojo.isArray(this.get('columns')['columnFilter']) || !dojo.isArray(this.get('columns')['columnsOrder']) ){
				return true;
		}
		
		if (dojo.isArray(this.get('columns')['columnList'])){
			var oldColumnList = this.get('columns')['columnList'];
			if (oldColumnList.length !== columnList.length) return true;

			for (var i=0; i<columnList.length; i++){
				if (columnList[i] !== oldColumnList[i]) return true;
			}
		}
		return false;
	},
	
    
    getViewerStyle:function(currentSource){
        return {
            titleValue: this.get('tableTitle'),
            titleFont: this.get('tableTitleFont'),
            titleAlign: this.get('tableTitleAlignment'),
            titleFontBold: this.get('tableTitleFontBold'),
            titleFontItalic: this.get('tableTitleFontItalic'),
            titleFontColor: this.get('tableTitleColor'),
            titleFontSize: this.get('tableTitleFontSize'),
            
            showHeader: this.get('showHeader'),
            headerFont: this.get('tableHeaderFont'),
            headerFontSize: this.get('tableHeaderFontSize'),
            headerFontBold: this.get('tableHeaderFontBold'),
            headerFontItalic: this.get('tableHeaderFontItalic'),
            headerFontColor: this.get('tableHeaderColor'),
            
            tableFont: this.get('tableContentFont'),
            tableFontSize: this.get('tableContentFontSize'),
            tableFontBold: this.get('tableContentFontBold'),
            tableFontItalic: this.get('tableContentFontItalic'),
            tableFontColor: this.get('tableContentColor'),
            
            columnLines: this.get('columnLineSize'),
            rowLines: this.get('rowLineSize'),
            oddRowColor: this.get('oddRowColor'),
            evenRowColor: this.get('evenRowColor'),
            
            highlightWithColor: this.get('highLightFilterColor'),
            columnsAlign: this.getColumnSettings(currentSource)['columnsAlign']
        };
    },
    
    getCaptionNode:function (){
            return {
					"fontFamily": this.get('tableTitleFont'),
					"textAlign": this.get('tableTitleAlignment'),
					"fontWeight": (this.get('tableTitleFontBold') ? "bold" : "normal"),                    
					"fontStyle": (this.get('tableTitleFontItalic') ? "italic" : "normal"),
					"color": this.get('tableTitleColor'),
					"fontSize": this.get('tableTitleFontSize'),
					"display": ""
				};
                
    },
    
    renderEditView:function(divFinder,dijitFinder,currentSource){
		divFinder("rowsPerPage").value = this.get('rowsPerView');
		divFinder("rowsLimit").value = this.get('rowsLimit');		
		divFinder("horizontalView").checked= this.get('horizontalView');
		divFinder("verticalView").checked=!this.get('horizontalView');
		divFinder("rowsLimit").disabled = this.get('horizontalView');
        
		dojo.style(divFinder("rowsLimitLabel"), "color", this.get('horizontalView') ? "#777777" : "#000000");
		
		divFinder("rowsPerPage").disabled = !this.get('horizontalView');;
		dojo.style(divFinder("rowsPerPageLabel"), "color", this.get('horizontalView') ? "#000000" : "#777777");
	
		divFinder("actionType_highlight").disabled = !this.get('horizontalView');
		divFinder("actionType_highlight").checked = this.get('horizontalView');
		divFinder("actionType_filter").checked = !this.get('horizontalView');
        
		
		var showHeaderCheckBox = divFinder("showHeader");
		showHeaderCheckBox.checked = this.get('showHeader');
        

		divFinder("tableTitle").value = this.get('tableTitle');
		divFinder("titleAlign").value = this.get('tableTitleAlignment');
		divFinder("titleFont").value = this.get('tableTitleFont');
		divFinder("titleFontSize").value = this.get('tableTitleFontSize');
		divFinder("titleFontBold").value = this.get('tableTitleFontBold');
		divFinder("titleFontBoldImg").src = (this.get('tableTitleFontBold')) ? this.contextRoot + "/images/bold_selected.png" : this.contextRoot + "/images/bold.png";
		divFinder("titleFontBoldImg").title = divFinder("titleFontBoldImg").alt = (this.get('tableTitleFontBold')) ? this.resourceBundle.L_CONF_STYLE_TABLE_TITLE_FONT_DISABLE_BOLD : this.resourceBundle.L_CONF_STYLE_TABLE_TITLE_FONT_ENABLE_BOLD;
		divFinder("titleFontItalic").value = this.get('tableTitleFontItalic');
		divFinder("titleFontItalicImg").src = (this.get('tableTitleFontItalic')) ? this.contextRoot + "/images/italic_selected.png" : this.contextRoot + "/images/italic.png";
		divFinder("titleFontItalicImg").title = divFinder("titleFontItalicImg").alt = (this.get('tableTitleFontItalic')) ? this.resourceBundle.L_CONF_STYLE_TABLE_TITLE_FONT_DISABLE_ITALIC : this.resourceBundle.L_CONF_STYLE_TABLE_TITLE_FONT_ENABLE_ITALIC;
		
		divFinder("headerFont").value = this.get('tableHeaderFont');
		divFinder("headerFontSize").value = this.get('tableHeaderFontSize');
		divFinder("headerFontBold").value = this.get('tableHeaderFontBold');
		divFinder("headerFontBoldImg").src = (this.get('tableHeaderFontBold')) ? this.contextRoot + "/images/bold_selected.png" : this.contextRoot + "/images/bold.png";
		divFinder("headerFontBoldImg").title = divFinder("headerFontBoldImg").alt = (this.get('tableHeaderFontBold')) ? this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_DISABLE_BOLD : this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_ENABLE_BOLD ;
		divFinder("headerFontItalic").value = this.get('tableHeaderFontItalic');
		divFinder("headerFontItalicImg").src = (this.get('tableHeaderFontItalic')) ? this.contextRoot + "/images/italic_selected.png" : this.contextRoot + "/images/italic.png";
		divFinder("headerFontItalicImg").title = divFinder("headerFontItalicImg").alt = (this.get('tableHeaderFontItalic')) ? this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_DISABLE_ITALIC : this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_ENABLE_ITALIC;

		divFinder("tableFont").value = this.get('tableContentFont');
		divFinder("tableFontSize").value = this.get('tableContentFontSize');
		divFinder("tableFontBold").value = this.get('tableContentFontBold');
		divFinder("tableFontBoldImg").src = (this.get('tableContentFontBold')) ? this.contextRoot + "/images/bold_selected.png" : this.contextRoot + "/images/bold.png";
		divFinder("tableFontBoldImg").title = divFinder("tableFontBoldImg").alt = (this.get('tableContentFontBold')) ? this.resourceBundle.L_CONF_STYLE_TABLE_FONT_DISABLE_BOLD : this.resourceBundle.L_CONF_STYLE_TABLE_FONT_ENABLE_BOLD;
		divFinder("tableFontItalic").value = this.get('tableContentFontItalic');
		divFinder("tableFontItalicImg").src = (this.get('tableContentFontItalic')) ? this.contextRoot + "/images/italic_selected.png" : this.contextRoot + "/images/italic.png";
		divFinder("tableFontItalicImg").title = divFinder("tableFontItalicImg").alt = (this.get('tableContentFontItalic')) ? this.resourceBundle.L_CONF_STYLE_TABLE_FONT_DISABLE_ITALIC : this.resourceBundle.L_CONF_STYLE_TABLE_FONT_ENABLE_ITALIC;

		dijitFinder(this.domID + "titleFontColor").attr("value", this.get('tableTitleColor'));
		dijitFinder(this.domID + "headerFontColor").attr("value", this.get('tableHeaderColor'));
		dijitFinder(this.domID + "tableFontColor").attr("value", this.get('tableContentColor'));
		dijitFinder(this.domID + "oddRowColor").attr("value", this.get('oddRowColor'));
		dijitFinder(this.domID + "evenRowColor").attr("value", this.get('evenRowColor'));

		var btn, img, align, str = "";
		dojo.forEach(["left", "center", "right"], function(align){
			img = divFinder('align_title_' + align + '_img');
			btn = divFinder('align_title_' + align + "_btn");
			img.src = this.contextRoot + "/images/align_" + align + ((this.get('tableTitleAlignment') === align) ? "_selected" : "") + ".png";
			if (this.get('tableTitleAlignment') === align) {
				str = dojo.string.substitute(this.resourceBundle.L_CONF_ALIGN_IS_SELECTED_1, [this.resourceBundle["L_" + align.toUpperCase()]]);
			} else {
				str = dojo.string.substitute(this.resourceBundle.L_CONF_ALIGN_1, [this.resourceBundle["L_" + align.toUpperCase()]]);
			}
			img.alt = btn.title = str;
			dojo.attr(img, "aria-label", str);

		}, this);
		divFinder("titleAlign").value = this.get('tableTitleAlignment');
		
		divFinder("columnLines").value = this.get('columnLineSize');
		divFinder("rowLines").value = this.get('rowLineSize');

		var columnAdSelect = divFinder("columnAd");
		dojo.empty(columnAdSelect);
		var optionValues = this.get('filterColumnOption');
		
		for (var i=0; i<optionValues.length; i++){
			var option = dojo.create("option");
			option.innerHTML = option.value = optionValues[i];
			if (option.value === this.settings['filterColumm']) {
				dojo.attr(option, "selected", "selected");
			}
			columnAdSelect.appendChild(option);
		}
		columnAdSelect.value = this.get('filterColumn');
		var contentTypeSelect = divFinder("contentType");
		
		var options = dojo.query("option[value='" + this.get('filterColumnType') + "')", contentTypeSelect);
		if (options.length > 0) {
			dojo.attr(options[0], "selected", "selected");
		}
		contentTypeSelect.value = this.get('filterColumnType');

		
		divFinder("comparVar").value = this.get('filterTargetValue');
		
		if (this.get('isFilter')) {
            divFinder("actionType_filter").checked=true;
		} else if (this.get('isHighLightFilter')) {
			divFinder("actionType_highlight").checked=true;
			dijitFinder(this.domID + "highlightWithColor").attr("disabled", false);
		} else if (this.get('isRemoveFilter')) {
			divFinder("actionType_remove").checked=true;
		}
		
		dojo.query("input[type=radio]", divFinder("filteringActionTypes")).onclick(dojo.hitch(this,function(event){
			dijitFinder(this.domID + "highlightWithColor").attr("disabled", !dojo.attr(divFinder("actionType_highlight"), "checked"));
		}));
		dijitFinder(this.domID + "highlightWithColor").attr("value", this.get('highLightFilterColor'));
		
		divFinder("invalid_feed").value = this.get('invalidFeedMessage');
		divFinder("no_content").value = this.get('noContentMessage');
		divFinder("all_content_filtered").value = this.get('allContentFilteredMessage');
		
    },
    getChangedValue:function(divFinder,dijitFinder, att, ColumnTable){
        var values={};
        values.rowsPerView = divFinder("rowsPerPage").value;
		values.rowsLimit = divFinder("rowsLimit").value;
		values.horizontalView = divFinder("horizontalView").checked;
		values.showHeader = divFinder("showHeader").checked;
		
		var columnSetting = null;
		if (ColumnTable) {
			columnSetting = ColumnTable.getSettings();
			values.columns = dojo.toJson(columnSetting);
		}
		this.columnManager.add(this.currentSource, columnSetting);
		
		values.columnHistory = this.columnManager.toJson();
		
        var dataViewerForm = divFinder("dataViewerForm");
        
		values.tableTitle = divFinder("tableTitle").value;
		values.tableTitleAlignment = divFinder("titleAlign").value;
		values.tableTitleFont = divFinder("titleFont").value;
		values.tableTitleFontSize = divFinder("titleFontSize").value;
		values.tableTitleFontBold = divFinder("titleFontBold").value;
		values.tableTitleFontItalic = divFinder("titleFontItalic").value;
		values.tableTitleColor = dijitFinder(this.domID + "titleFontColor").attr("value");
		
		values.tableHeaderFont = divFinder("headerFont").value;
		values.tableHeaderFontSize = divFinder("headerFontSize").value;
		values.tableHeaderFontBold = divFinder("headerFontBold").value;
		values.tableHeaderFontItalic = divFinder("headerFontItalic").value;
		values.tableHeaderColor = dijitFinder(this.domID + "headerFontColor").attr("value");

		values.tableContentFont = divFinder("tableFont").value;
		values.tableContentFontSize = divFinder("tableFontSize").value;
		values.tableContentFontBold = divFinder("tableFontBold").value;
		values.tableContentFontItalic = divFinder("tableFontItalic").value;
		values.tableContentColor = dijitFinder(this.domID + "tableFontColor").attr("value");

		values.columnLineSize = divFinder("columnLines").value;
		values.rowLineSize = divFinder("rowLines").value;
		values.oddRowColor = dijitFinder(this.domID + "oddRowColor").attr("value");
		values.evenRowColor = dijitFinder(this.domID + "evenRowColor").attr("value");

		values.filterColumn = divFinder("columnAd").value;
		var optArray = dojo.query("option", divFinder("columnAd")).map(function(opt) {
			return opt.value;
		}, this);
		values.filterColumnOption = dojo.toJson(optArray);
		
		values.filterColumnType = divFinder("contentType").value;
		values.filterFunction = divFinder("contentCompare").value;
		values.filterTargetValue = divFinder("comparVar").value || "";
		
		values.filterType = "";
		var filterInputs = dataViewerForm['actionType'];
		dojo.forEach(filterInputs, function(radio) {
			if (radio.checked) values.filterType = radio.value;
		}, this);
		
		values.isHighLightFilter = "" + (values.filterType === 'highlight');
		values.isFilter = "" + (values.filterType === 'filter');
		values.isRemoveFilter = "" + (values.filterType === 'remove');
		values.highLightFilterColor = dijitFinder(this.domID + "highlightWithColor").attr("value");
		values.invalidFeedMessage = dojo.trim(dataViewerForm['invalidFeed'].value) || com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages.E_INVALID_ATOM_FEED;
		values.noContentMessage = dojo.trim(dataViewerForm['noContent'].value) || com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages.E_NO_CONTENT;
		values.allContentFilteredMessage = dojo.trim(dataViewerForm['allContentFiltered'].value) || com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages.I_ALL_CONTENT_FILTERED;
        
        var changedVals ={};
        for(var name in values){
            if(att.getItemValue(name)+'' != values[name]+''){
                if(this.isEmpty(att.getItemValue(name)) && !this.isDefaultValue(name,values[name])){
                    changedVals[name] =values[name];
                }else if(!this.isEmpty(att.getItemValue(name))){
                    changedVals[name] =values[name];
                }
            }
        }
        //for 20330. filterColumnType and filterTargetValue cannot be undefined since they are used to check if numeric operations have both numeric numbers.
        changedVals.filterColumnType = values.filterColumnType;
        changedVals.filterTargetValue = values.filterTargetValue;
        return changedVals;

    },
    
    isEmpty:function(value){
        return !value || value=='';
    }
    
    
    
});
