/* Copyright IBM Corp. 2009, 2010  All Rights Reserved.                    */
dojo.provide("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.VerticalDataViewer");

/**
 * JavaScript for dataViewer
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojo.number");
dojo.require("dojox.color");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.Store");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil");

dojo.declare("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.VerticalDataViewer", [com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataViewer], {

    postMixInProperties: function() {
		this.inherited(arguments);
		this.store = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.Store();
	},

    createRows: function(/* object */obj, index){
        //	summary
		var rows = [];
		dojo.forEach(this.columns, dojo.hitch(this, function(col, j) {
		
			//	Create an HTML row based on the passed object
			var row = dojo.create("tr", {
				value: index,
				idx: j
			});
			
			var ds = this.dataViewerStyle,
				hc = ds.highlightWithColor,
				ihc = this._getInvertHighContrastColor(hc),
				ish = this.isRowHighlighted(row, index),
//				isS = this.isRowSelected(row),
				self = this;

			// IE6 doesn't support :hover on <tr> element
			if (dojo.isIE <= 6) {
				(function (scope, row, rIndex, isHighlighted){
					self._rowConnects.push(dojo.connect(row, "onmouseover", function(){
						this.style.backgroundColor = "#d0cedf";//from dataviewer.css
					}));
					self._rowConnects.push(dojo.connect(row, "onmouseout", function(){
						var bgColor = isHighlighted ? hc : ((j % 2 === 1) ? scope.dataViewerStyle.evenRowColor : scope.dataViewerStyle.oddRowColor);
						dojo.style(this, {
							backgroundColor: bgColor
						});
					}));
				})(this, row, index, ish);
			}
			
//			dojo.toggleClass(row, this.rowSelectedClass, this.isRowSelected(row));
			dojo.toggleClass(row, this.rowHighlightClass, !!ish);

			dojo.style(row, {
				backgroundColor: ish ? hc : ((j % 2 === 1) ? ds.evenRowColor : ds.oddRowColor)
//				color: ish || isS ? ihc : ds.tableFontColor,
			});
		
			if (this.dataViewerStyle.showHeader){
			
				var cell1 = dojo.create("td", {
					innerHTML: ["<span tabindex=0>", (col.label || ""), "</span>"].join("")				
				}, row);
				
				dojo.style(cell1, {
					borderLeftWidth: ds.columnLines,
					borderRightWidth: ds.columnLines,
					textAlign: "right",
					fontWeight: ds.headerFontBold ? "bold" : "",				
					fontStyle: (ds.headerFontItalic) ? "italic" : "",				
					color: ds.headerFontColor,
					fontFamily: ds.headerFont,
					fontSize: ds.headerFontSize,
					borderTopWidth: ds.rowLines,
					borderBottomWidth: ds.rowLines
				})
			}
			var cell2 = dojo.create("td", {
				innerHTML: ["<span tabindex=0>", (obj.src[col.getField()] || ""), "</span>"].join("")
			}, row);
			
			
			dojo.style(cell2, {
				borderLeftWidth: ds.columnLines,
				borderRightWidth: ds.columnLines,
				textAlign: ds.columnsAlign[col.field] || "left",
				fontWeight: ds.tableFontBold ? "bold" : "",				
				fontStyle: (ds.tableFontItalic) ? "italic" : "",				
				color: ds.tableFontColor,
				fontFamily: ds.tableFont,
				fontSize: ds.tableFontSize,
				borderTopWidth: ds.rowLines,
				borderBottomWidth: ds.rowLines
			})
			
			rows.push(row);
			
		}));
		
        return rows; //	HTMLTableRow
    },
    
    renderRow: function(start, length){
		
        var data = this.store.get();
		var ds = this.dataViewerStyle;
		
        this.clearRow();

        var body = this.tableNode.tBodies[0];
		dojo.style(body, {
			fontFamily: ds.tableFont,
			fontSize: ds.tableFontSize,
			fontStyle: (ds.tableFontItalic) ? "italic" : "",
			fontWeight: (ds.tableFontBold) ? "bold" : ""
		});
		
        try{
	        for (var i = start; i < start + length && i < data.length; i++) {
	            var rows = this.createRows(data[i], i);
				
				dojo.forEach(rows, function(row){
					body.appendChild(row);
				});
	            
	        }
        } catch(e){;}
    },
    

    renderTable: function(){
        //	summary
        //	render table title, summary, header. Do not render table body
		
        //For a11y
		var resourceBundle = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages;
		dojo.attr(this.tableNode, "summary", this.dataViewerStyle.titleValue ? this.dataViewerStyle.titleValue : resourceBundle.I_TABLE_DEFAULT_SUMMARY);

        //	if there is no thead, create it now.
        var head = this.tableNode.getElementsByTagName("thead")[0];
		
        if (head.getElementsByTagName("tr").length === 0) {
            var row = dojo.create("tr");
            
			var cell = dojo.create("th", {				
				align: "left",
				scope: "colgroup",
				colspan: "2",
				innerHTML: "<span tabindex=0>" + this.dataViewerStyle.titleValue + "</span>"				
			});
			
			// dojo.addClass(cell, this.headerClass);
			dojo.style(cell, {
				fontStyle: (this.dataViewerStyle.titleFontItalic) ? "italic" : "",
				fontWeight: (this.dataViewerStyle.titleFontBold) ? "bold" : "",
				color: this.dataViewerStyle.titleFontColor,
				fontFamily: this.dataViewerStyle.titleFont,
				fontSize: this.dataViewerStyle.titleFontSize,
				textAlign: this.dataViewerStyle.titleAlign
			});
			

			row.appendChild(cell);
            head.appendChild(row);
        }
		
        //No sorting column at default
        this.sortInformation = [];

        this.store.forEach(function(element){
			element.isSelected = false;
		});

	},
    
	renderFirstPage: function() {
		this.store.setData(this.data);		
		this.inherited(arguments);
	},
	
	onLoad : function(e) {
		
		this.store = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.Store();
		this.store.keyField = this.valueField;
		
		this.inherited(arguments);		
		this.domNode.appendChild(this.tableNode);
	},

	//add lablels as input parameter
	fetchAllInfo: function(data, filter, lables, sorttype){
		
		if (data.result.length > 0 && data.columnList.length > 0) {
	        this.onLoad();

	        this.paginationData = data;
	        
	        this.columnFilter = filter;
            
            this.typeList = []
			for (var i=0; i<data.columnList.length; i++){
				this.typeList.push('text');
			}

            this.parseColumns(data, lables, sorttype);
            
            this.data = data.result;
        }
		
    },

	onDataSelected: function(/*object*/data, /*number*/ columnIndex){
        var rowDataColumn = "",
            rowDataType = "",
            rowDataValue = "",
            rowData = "",
            cellValue = "";
        
        if (this.columns.length <= 0) 
            return;

        rowData = this.columns[columnIndex].label + "|" + this.columns[columnIndex].dataType + "|" + data[this.columns[columnIndex].label];
        
		var celljsonData = {};
		var cellKey = this.columns[columnIndex].field;
		celljsonData[cellKey] = data[cellKey];
				
		cellValue = data[cellKey];
		
		var dataInfo = {
                widgetId: this.mmWidgetId,
                rowData: rowData,
                cellValue: cellValue,
                rowJSonData: celljsonData,
				cellJSonData: celljsonData
		};
        dojo.publish("dataViewer/" + this.mmWidgetId + "/dataSelected", [{dataInfo: dataInfo}]);
    },
    
	onSelectCell: function(/* HTMLEvent */e){
		this.selectedColumnIndex = -1;
		var cell = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil.getParentByType(e.target,"td");
		if (cell) {
			var row = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil.getParentByType(cell,"tr");
			this.selectedColumnIndex = parseInt(dojo.attr(row, "idx"), 10);
		}
	},

	renderSelections: function(row){
		//	summary
		//	Render all selected objects using CSS.
		var body=this.tableNode.tBodies[0];
		
		// clear old selection
		dojo.query("tr.selected td", body).style({
			color: this.dataViewerStyle.tableFontColor
		});
		
		for(var i=0, l=body.rows.length; i<l; i++){			
			dojo.toggleClass(body.rows[i], this.rowSelectedClass, body.rows[i] === row);
		}
		var ihc = this._getInvertHighContrastColor(this.dataViewerStyle.highlightWithColor);
		dojo.query("tr.selected td, tr.highlighted td", body).style({
			color: ihc
		});
	},

	//	standard events
	onSelect: function(/* HTMLEvent */e){
		//	summary
		//	Handles the onclick event of any element.
		var row = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil.getParentByType(e.target,"tr");

		this.resetSelections();
		this.toggleSelectionByRow(row);
		
		this.renderSelections(row);
		
		if (this.allowCellSelection) {
			this.onSelectCell(e);
			
		}
		
		this.onDataSelected(this.getDataByRow(row), this.selectedColumnIndex);
		
	}
	
});
