-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
-- Run the script in the DB2 command line processor with admin privileges.
--
-- USAGE:   db2 -tf preMigrateBusinessSpace700.sql
--
-- ------------------------------------------------------------------------------------

-- Must replace value of @VCAT@
CREATE STOGROUP BSPMT VOLUMES ('*') VCAT BSPMT;

CREATE TABLESPACE BMTRTP1 
	IN @DB_NAME@
 	USING STOGROUP BSPMT
 	CCSID UNICODE
 	SEGSIZE 32
 	DEFINE NO
 	LOCKSIZE ANY
 	BUFFERPOOL BP32K;                     

CREATE LOB TABLESPACE BMTLTP1
 	IN @DB_NAME@
 	USING STOGROUP BSPMT; 

CREATE LOB TABLESPACE BMTLTP2
 	IN @DB_NAME@
 	USING STOGROUP BSPMT; 

CREATE LOB TABLESPACE BMTLTP3
 	IN @DB_NAME@
 	USING STOGROUP BSPMT; 

CREATE LOB TABLESPACE BMTLTP4
 	IN @DB_NAME@
 	USING STOGROUP BSPMT; 


-- *****************************************************************
--
-- SCHEMA VERSION (100)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.VERSION_INFO(
    OID VARCHAR(64)    NOT NULL,
    VERSION_MAJOR   INTEGER,
    VERSION_MINOR   INTEGER,
    VERSION_RELEASE INTEGER,
    VERSION_DEV     INTEGER,
    VERSION_TIME    DECIMAL(19,0),
    DESCRIPTION     VARCHAR(254),
    CONSTRAINT PK100 PRIMARY KEY (OID)
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.VERSION_INFO_PK
	ON BSPMGTMP.VERSION_INFO (OID) 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- CATALOG (300)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG (
    OID         VARCHAR(64)       NOT NULL,
    ACCOUNT_OID VARCHAR(1024)     NOT NULL,
    UNIQUE_NAME VARCHAR(255)      NOT NULL,
    CREATED     DECIMAL(19,0)            NOT NULL,
    MODIFIED    DECIMAL(19,0)            NOT NULL,
    STATE       INTEGER DEFAULT 1 NOT NULL,
    CONSTRAINT PK300 PRIMARY KEY (OID)
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_PK
	ON BSPMGTMP.CATALOG (OID) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.CATALOG_LOD (
    RESOURCE_OID VARCHAR(64)       NOT NULL,
    LOCALE       VARCHAR(64)    NOT NULL,
    TITLE        VARCHAR(255)   NOT NULL,
    DESCRIPTION  VARCHAR(1024)  ,
    CONSTRAINT PK301 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK302 FOREIGN KEY (RESOURCE_OID) REFERENCES
      BSPMGTMP.CATALOG (OID) ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_LOD_PK
	ON BSPMGTMP.CATALOG_LOD (RESOURCE_OID, LOCALE) 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- CATALOG ENTRY (310)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_ENTRY (
    OID            			VARCHAR(64)        NOT NULL,
    CATALOG_OID    			VARCHAR(64)        NOT NULL,
    CATEGORY_OID   			VARCHAR(64)        NOT NULL,
    DEFINITION_URL 			VARCHAR(2500)      NOT NULL,
    CONTENT_URL    			VARCHAR(2500)      ,
    PREVIEW_URL    			VARCHAR(2500)      ,
    PREVIEW_THUMBNAIL_URL	VARCHAR(2500)      ,
    HELP_URL       			VARCHAR(2500)      ,    
    ICON_URL       			VARCHAR(2500)      ,
    HUB_ENTRY_URL  			VARCHAR(2500)      ,
    DEF_URL_TYPE   			INTEGER            ,
    ICON_URL_TYPE  			INTEGER            ,
    CREATED        			DECIMAL(19,0)             NOT NULL,
    MODIFIED       			DECIMAL(19,0)             NOT NULL,
    STATE          			INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK310 PRIMARY KEY (OID)
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_ENTRY_PK
	ON BSPMGTMP.CATALOG_ENTRY (OID) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.CATALOG_ENTRY_LOD (
    RESOURCE_OID  		VARCHAR(64)    NOT NULL,
    LOCALE        		VARCHAR(64)    NOT NULL,
    TITLE         		VARCHAR(255)   NOT NULL,
    DESCRIPTION   		VARCHAR(1024)  ,
    SHORT_DESCRIPTION   VARCHAR(255)   ,    
    CONSTRAINT PK311 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK311 FOREIGN KEY (RESOURCE_OID) REFERENCES
      BSPMGTMP.CATALOG_ENTRY (OID) ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_ENTRY_LOD_PK
	ON BSPMGTMP.CATALOG_ENTRY_LOD (RESOURCE_OID, LOCALE) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.CATALOG_ENTRY_DD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    NAME          VARCHAR(255)   NOT NULL,
    VALUE         VARCHAR(3000)  NOT NULL,
    CONSTRAINT PK312 PRIMARY KEY (RESOURCE_OID, NAME),
    CONSTRAINT FK312 FOREIGN KEY (RESOURCE_OID) REFERENCES
      BSPMGTMP.CATALOG_ENTRY (OID) ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_ENTRY_DD_PK
	ON BSPMGTMP.CATALOG_ENTRY_DD (RESOURCE_OID, NAME) 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- CATALOG CATEGORY (320)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_CATEGORY (
    OID         VARCHAR(64)         NOT NULL,
    CATALOG_OID VARCHAR(64)         NOT NULL,
    NAME        VARCHAR(255)        ,
    CREATED     DECIMAL(19,0)              NOT NULL,
    MODIFIED    DECIMAL(19,0)              NOT NULL,
    STATE       INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK320 PRIMARY KEY (OID),
    CONSTRAINT FK320 FOREIGN KEY (CATALOG_OID) REFERENCES
      BSPMGTMP.CATALOG (OID) ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_CATEGORY_PK
	ON BSPMGTMP.CATALOG_CATEGORY (OID) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.CATALOG_CATEGORY_LOD (
    RESOURCE_OID VARCHAR(64)    NOT NULL,
    LOCALE       VARCHAR(64)    NOT NULL,
    TITLE        VARCHAR(255)   NOT NULL,
    DESCRIPTION  VARCHAR(1024)  ,
    CONSTRAINT PK321 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK321 FOREIGN KEY (RESOURCE_OID) REFERENCES
      BSPMGTMP.CATALOG_CATEGORY (OID) ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_CATEGORY_LOD_PK
	ON BSPMGTMP.CATALOG_CATEGORY_LOD (RESOURCE_OID, LOCALE) 
	USING STOGROUP BSPMT;


-- *****************************************************************
--
-- CATALOG CATEGORY INCLUDES (330)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_CATEGORY_INC (
    OID          		VARCHAR(64)       NOT NULL,
    CATALOG_OID  		VARCHAR(64)       NOT NULL,
    CATEGORY_OID 		VARCHAR(64)       NOT NULL,
    INCLUDE_CAT  		VARCHAR(255)  	  NOT NULL,
    INCLUDE_CATEGORY	VARCHAR(255)      NOT NULL,
    CONSTRAINT PK330 PRIMARY KEY (OID)
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_CATEGORY_INC_PK
	ON BSPMGTMP.CATALOG_CATEGORY_INC (OID) 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- CATALOG INCLUDES (340)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_INC (
    OID          		VARCHAR(64)       NOT NULL,
    CATALOG_OID  		VARCHAR(64)       NOT NULL,
    INCLUDE_CAT  		VARCHAR(255)  	  NOT NULL,
    CONSTRAINT PK340 PRIMARY KEY (OID)
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.CATALOG_INC_PK
	ON BSPMGTMP.CATALOG_INC (OID) 
	USING STOGROUP BSPMT;
	
-- *****************************************************************
--
-- PREFERENCE (400)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.PREFERENCE (
    OID          VARCHAR(64)        NOT NULL,
    UNIQUE_NAME  VARCHAR(255)       NOT NULL,
    PARENT_OID   VARCHAR(64)        NOT NULL,
    TYPE         INTEGER            NOT NULL,
    WDGT_INST_ID VARCHAR(255)       NOT NULL,
    ACCOUNT_ID   VARCHAR(1024)      NOT NULL,
    PAGE_ID      VARCHAR(255)       NOT NULL,
    CREATED      DECIMAL(19,0)             NOT NULL,
    MODIFIED     DECIMAL(19,0)             NOT NULL,
    STATE        INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK401 PRIMARY KEY (OID)
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.PREFERENCE_PK
	ON BSPMGTMP.PREFERENCE (OID) 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- PREFERENCE NODE VALUE (410)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.PREFERENCE_VALUE (
    OID           VARCHAR(64)        NOT NULL,
    PREF_OID      VARCHAR(64)        NOT NULL,
    NAME          VARCHAR(255)       NOT NULL,
    READ_ONLY     INTEGER DEFAULT 0  NOT NULL,
    IS_LARGE      INTEGER DEFAULT 0  NOT NULL,
    HAS_LIST      INTEGER DEFAULT 0  NOT NULL,
    VALUE         VARCHAR(1024)      ,
    LARGE_VALUE   CLOB(1M)           ,
    STATE         INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK410 PRIMARY KEY (OID),
    CONSTRAINT FK410 FOREIGN KEY (PREF_OID) REFERENCES
      BSPMGTMP.PREFERENCE (OID) ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.PREFERENCE_VALUE_PK
	ON BSPMGTMP.PREFERENCE_VALUE (OID) 
	USING STOGROUP BSPMT;

CREATE AUX TABLE BSPMGTMP.LARGE_VALUE_LOB_A
	IN @DB_NAME@.BMTLTP1 
	STORES BSPMGTMP.PREFERENCE_VALUE 
	COLUMN LARGE_VALUE;

CREATE INDEX BSPMGTMP.LARGE_VALUE_LOB_I 
	ON BSPMGTMP.LARGE_VALUE_LOB_A 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- NAVIGATION NODE (500)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.NAVNODE (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_OID      VARCHAR(1024)      NOT NULL,
    THEME_ID       VARCHAR(255)       NOT NULL,
    THEME_NAME     VARCHAR(255)       NOT NULL,
    CONTENT        BLOB(1M)           ,
    BYTE_COUNT     INTEGER            ,
    IS_ZIPPED      INTEGER DEFAULT 0  NOT NULL,
    COMMUNITY_OID  VARCHAR(64)        ,
    SPACE_OID      VARCHAR(64)        ,
    CREATED        DECIMAL(19,0)             NOT NULL,
    MODIFIED       DECIMAL(19,0)             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK500 PRIMARY KEY (OID, OWNER_OID)
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.NAVNODE_PK
	ON BSPMGTMP.NAVNODE (OID, OWNER_OID) 
	USING STOGROUP BSPMT;

CREATE AUX TABLE BSPMGTMP.CONTENT_LOB_A
	IN @DB_NAME@.BMTLTP2 
	STORES BSPMGTMP.NAVNODE 
	COLUMN CONTENT;

CREATE INDEX BSPMGTMP.CONTENT_LOB_I 
	ON BSPMGTMP.CONTENT_LOB_A
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.NAVNODE_LOD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    OWNER_OID     VARCHAR(1024)  NOT NULL,
    LOCALE        VARCHAR(64)    NOT NULL,
    TITLE         VARCHAR(255)   NOT NULL,
    DESCRIPTION   VARCHAR(1024)          ,
    CONSTRAINT FK500 FOREIGN KEY (RESOURCE_OID,OWNER_OID)
      REFERENCES BSPMGTMP.NAVNODE (OID,OWNER_OID) 
        ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1
;

CREATE TABLE BSPMGTMP.NAVNODE_DD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    OWNER_OID     VARCHAR(1024)  NOT NULL,
    NAME          VARCHAR(255)   NOT NULL,
    VALUE         VARCHAR(3000)  NOT NULL,
    CONSTRAINT PK502 PRIMARY KEY (RESOURCE_OID,OWNER_OID,NAME),
    CONSTRAINT FK502 FOREIGN KEY (RESOURCE_OID,OWNER_OID)
      REFERENCES BSPMGTMP.NAVNODE (OID,OWNER_OID)
        ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1;

CREATE UNIQUE INDEX BSPMGTMP.NAVNODE_DD_PK
	ON BSPMGTMP.NAVNODE_DD (RESOURCE_OID,OWNER_OID,NAME) 
	USING STOGROUP BSPMT;


-- *****************************************************************
--
-- NAVIGATION TREE (510)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.NAV_TREE (
    OWNER_OID    VARCHAR(1024)       NOT NULL,
    PAGE_OID     VARCHAR(64)         NOT NULL,
    PARENT_OID   VARCHAR(64)         NOT NULL,
    ORDINAL      INTEGER  DEFAULT 1  ,
    IS_VIRTUAL   INTEGER  DEFAULT 0  ,
    CONSTRAINT PK510 PRIMARY KEY (OWNER_OID, PAGE_OID)
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.NAV_TREE_PK
	ON BSPMGTMP.NAV_TREE (OWNER_OID, PAGE_OID) 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- ACCESS CONTROL (600)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.COMMUNITY_DEF (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_UID      VARCHAR(1024)      NOT NULL,
    RESOURCE_OID   VARCHAR(64)        NOT NULL,
    RESOURCE_TYPE  VARCHAR(32)        NOT NULL,
    CREATED        DECIMAL(19,0)             NOT NULL,
    MODIFIED       DECIMAL(19,0)             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK600 PRIMARY KEY( OID )
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.COMMUNITY_DEF_PK
	ON BSPMGTMP.COMMUNITY_DEF (OID) 
	USING STOGROUP BSPMT;

-- UNIQUE CONSTRAINT
-- ALTER TABLE BSPMGTMP.COMMUNITY_DEF
-- 	 ADD CONSTRAINT UQ600 UNIQUE (OID,RESOURCE_OID);

CREATE TABLE BSPMGTMP.ACL (
    COMMUNITY_OID     VARCHAR(64)        NOT NULL,
    PARTICIPANT_UID   VARCHAR(1024)      NOT NULL,
    PARTICIPANT_TYPE  INTEGER            NOT NULL,
    PERMISSIONS       VARCHAR(255)       NOT NULL,
    CREATED           DECIMAL(19,0)             NOT NULL,
    MODIFIED          DECIMAL(19,0)             NOT NULL,
    STATE             INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK610 PRIMARY KEY (COMMUNITY_OID,PARTICIPANT_UID),
    CONSTRAINT FK611 FOREIGN KEY (COMMUNITY_OID)
      REFERENCES BSPMGTMP.COMMUNITY_DEF (OID)
        ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.ACL_PK
	ON BSPMGTMP.ACL (COMMUNITY_OID,PARTICIPANT_UID) 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- SPACES (700)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.SPACENODE (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_OID      VARCHAR(1024)      NOT NULL,
    ROOTNAV_OID    VARCHAR(64)        ,
    TEMPLATE_OID   VARCHAR(64)        ,
    CREATED        DECIMAL(19,0)             NOT NULL,
    MODIFIED       DECIMAL(19,0)             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK700 PRIMARY KEY (OID, OWNER_OID)
)  IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.SPACENODE_PK
	ON BSPMGTMP.SPACENODE (OID, OWNER_OID) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.SPACENODE_LOD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    OWNER_OID     VARCHAR(1024)  NOT NULL,
    LOCALE        VARCHAR(64)    NOT NULL,
    TITLE         VARCHAR(255)   NOT NULL,
    DESCRIPTION   VARCHAR(1024)          ,
    CONSTRAINT PK701 PRIMARY KEY (RESOURCE_OID,OWNER_OID,LOCALE),
    CONSTRAINT FK701 FOREIGN KEY (RESOURCE_OID,OWNER_OID)
      REFERENCES BSPMGTMP.SPACENODE (OID,OWNER_OID)
        ON DELETE CASCADE
)  IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.SPACENODE_LOD_PK
	ON BSPMGTMP.SPACENODE_LOD (RESOURCE_OID,OWNER_OID,LOCALE) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.SPACENODE_DD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    OWNER_OID     VARCHAR(1024)  NOT NULL,
    NAME          VARCHAR(255)   NOT NULL,
    VALUE         VARCHAR(3000)  NOT NULL,
    CONSTRAINT PK702 PRIMARY KEY (RESOURCE_OID,OWNER_OID,NAME),
    CONSTRAINT FK702 FOREIGN KEY (RESOURCE_OID,OWNER_OID)
      REFERENCES BSPMGTMP.SPACENODE (OID,OWNER_OID)
        ON DELETE CASCADE
)  IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.SPACENODE_DD_PK
	ON BSPMGTMP.SPACENODE_DD (RESOURCE_OID,OWNER_OID,NAME) 
	USING STOGROUP BSPMT;


-- *****************************************************************
--
-- TEMPLATES (800)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.TEMPLATENODE (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_OID      VARCHAR(1024)      ,
    CONTENT        BLOB(1M)           ,
    BYTE_COUNT     INTEGER            ,
    TEMPLATE_TYPE  VARCHAR(64)        ,
    CREATED        DECIMAL(19,0)             NOT NULL,
    MODIFIED       DECIMAL(19,0)             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK800 PRIMARY KEY (OID)
)  IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.TEMPLATENODE_PK
	ON BSPMGTMP.TEMPLATENODE (OID) 
	USING STOGROUP BSPMT;

CREATE AUX TABLE BSPMGTMP.CONTENT_LOB_B
	IN @DB_NAME@.BMTLTP4 
	STORES BSPMGTMP.TEMPLATENODE 
	COLUMN CONTENT;

CREATE INDEX BSPMGTMP.CONTENT_LOB_J 
	ON BSPMGTMP.CONTENT_LOB_B
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.TEMPLATENODE_LOD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    LOCALE        VARCHAR(64)    NOT NULL,
    TITLE         VARCHAR(255)   NOT NULL,
    DESCRIPTION   VARCHAR(1024)          ,
    CONSTRAINT PK801 PRIMARY KEY (RESOURCE_OID,LOCALE),
    CONSTRAINT FK801 FOREIGN KEY (RESOURCE_OID)
      REFERENCES BSPMGTMP.TEMPLATENODE (OID)
        ON DELETE CASCADE
)  IN @DB_NAME@.BMTRTP1
;

CREATE UNIQUE INDEX BSPMGTMP.TEMPLATENODE_LOD_PK
	ON BSPMGTMP.TEMPLATENODE_LOD (RESOURCE_OID,LOCALE) 
	USING STOGROUP BSPMT;


-- *****************************************************************
--
-- ACCESS CONTROL RESOURCES (900)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.AC_RESOURCE (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_UID      VARCHAR(1024)      NOT NULL,
    RESOURCE_OID   VARCHAR(64)        NOT NULL,
    RESOURCE_TYPE  VARCHAR(32)        NOT NULL,
    ROLE_BLOCKS    VARCHAR(1024)              ,
    CREATED        DECIMAL(19,0)             NOT NULL,
    MODIFIED       DECIMAL(19,0)             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK900 PRIMARY KEY( RESOURCE_OID )
) IN @DB_NAME@.BMTRTP1;

CREATE UNIQUE INDEX BSPMGTMP.AC_RESOURCE_PK
	ON BSPMGTMP.AC_RESOURCE (RESOURCE_OID) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.AC_ROLE (
    AC_RESOURCE_OID   VARCHAR(64)        NOT NULL,
    ROLE_OID          VARCHAR(64)        NOT NULL,
    ROLE_NAME         VARCHAR(1024)      NOT NULL,
    ROLE_TYPE         VARCHAR(1024)              ,
    CREATED        DECIMAL(19,0)             NOT NULL,
    MODIFIED       DECIMAL(19,0)             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK910 PRIMARY KEY( ROLE_OID ),
    CONSTRAINT FK911 FOREIGN KEY (AC_RESOURCE_OID)
      REFERENCES BSPMGTMP.AC_RESOURCE (RESOURCE_OID)
        ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1;

CREATE UNIQUE INDEX BSPMGTMP.AC_ROLE_PK
	ON BSPMGTMP.AC_ROLE (ROLE_OID) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.AC_MEMBER (
    ROLE_OID          VARCHAR(64)        NOT NULL,
    MEMBER_UID        VARCHAR(1024)      NOT NULL,
    MEMBER_TYPE       DECIMAL(19,0)            NOT NULL,
    CONSTRAINT PK920 PRIMARY KEY (ROLE_OID,MEMBER_UID),
    CONSTRAINT FK921 FOREIGN KEY (ROLE_OID)
      REFERENCES BSPMGTMP.AC_ROLE (ROLE_OID)
        ON DELETE CASCADE
) IN @DB_NAME@.BMTRTP1;

CREATE UNIQUE INDEX BSPMGTMP.AC_MEMBER_PK
	ON BSPMGTMP.AC_MEMBER (ROLE_OID,MEMBER_UID) 
	USING STOGROUP BSPMT;

-- *****************************************************************
--
-- FILESTORE (1000)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.FILESTORE_PATH (
    OID         VARCHAR(64)           NOT NULL,
    PARENT_OID  VARCHAR(64)           NOT NULL,
    NAME        VARCHAR(255)          NOT NULL,
    PATH        VARCHAR(255)          NOT NULL,
    LOCKED      VARCHAR(64)           , 
    OWNER       VARCHAR(64)           ,             
    CREATED     DECIMAL(19,0)                NOT NULL,
    MODIFIED    DECIMAL(19,0)                NOT NULL,
    COLL_TYPE   DECIMAL(19,0) DEFAULT 1     NOT NULL,
    CONSTRAINT PK1010 PRIMARY KEY (OID)
) IN @DB_NAME@.BMTRTP1;

CREATE UNIQUE INDEX BSPMGTMP.FILESTORE_PATH_PK
	ON BSPMGTMP.FILESTORE_PATH (OID) 
	USING STOGROUP BSPMT;

CREATE TABLE BSPMGTMP.FILESTORE_FILE (
    OID         VARCHAR(64)           NOT NULL,           
    F_SIZE      DECIMAL(19,0)              NOT NULL,
    CONTENT        BLOB(1G)           ,
    CONSTRAINT PK1000 PRIMARY KEY (OID),
    CONSTRAINT FK1001 FOREIGN KEY (OID)
      REFERENCES BSPMGTMP.FILESTORE_PATH (OID)
        ON DELETE CASCADE
)  IN @DB_NAME@.BMTRTP1;

CREATE UNIQUE INDEX BSPMGTMP.FILESTORE_FILE_PK
	ON BSPMGTMP.FILESTORE_FILE (OID) 
	USING STOGROUP BSPMT;

CREATE AUX TABLE BSPMGTMP.CONTENT_LOB_C
	IN @DB_NAME@.BMTLTP3 
	STORES BSPMGTMP.FILESTORE_FILE 
	COLUMN CONTENT;

CREATE INDEX BSPMGTMP.CONTENT_LOB_K 
	ON BSPMGTMP.CONTENT_LOB_C
	USING STOGROUP BSPMT;

--*****************************************************************
--
-- Indexing
--
--*****************************************************************
-- Catalog
CREATE INDEX BSPMGTMP.CATACCT      ON 
  BSPMGTMP.CATALOG(ACCOUNT_OID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.ENTRYCATGOID ON 
  BSPMGTMP.CATALOG_ENTRY(CATALOG_OID, CATEGORY_OID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.CATGCATOID   ON 
  BSPMGTMP.CATALOG_CATEGORY(CATALOG_OID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.CATINTCATOID ON 
  BSPMGTMP.CATALOG_INC(CATALOG_OID) 
    USING STOGROUP BSPMT;

-- Preferences
CREATE INDEX BSPMGTMP.PREFPARENT ON 
  BSPMGTMP.PREFERENCE(PARENT_OID,ACCOUNT_ID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.PREFTYPE   ON 
  BSPMGTMP.PREFERENCE(TYPE) 
    USING STOGROUP BSPMT;

-- Navigation
CREATE INDEX BSPMGTMP.PAGECOMM     ON 
  BSPMGTMP.NAVNODE(COMMUNITY_OID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.PAGELODOWNRS ON 
  BSPMGTMP.NAVNODE_LOD(OWNER_OID,RESOURCE_OID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.TREEOWPAR    ON 
  BSPMGTMP.NAV_TREE(OWNER_OID,PARENT_OID) 
    USING STOGROUP BSPMT;

-- Acl
CREATE INDEX BSPMGTMP.COMMOWNPAR ON 
  BSPMGTMP.COMMUNITY_DEF(OWNER_UID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.PRTTYUID   ON 
  BSPMGTMP.ACL(PARTICIPANT_TYPE,PARTICIPANT_UID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.PRTUID     ON 
  BSPMGTMP.ACL(PARTICIPANT_UID) 
    USING STOGROUP BSPMT;

-- AC
CREATE INDEX BSPMGTMP.AC_ROLE_RESOURCE ON 
  BSPMGTMP.AC_ROLE(AC_RESOURCE_OID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.AC_MEMBER_UID ON 
  BSPMGTMP.AC_MEMBER(MEMBER_UID) 
    USING STOGROUP BSPMT;
CREATE INDEX BSPMGTMP.AC_RESOURCE_OWNER ON 
  BSPMGTMP.AC_RESOURCE(OWNER_UID,RESOURCE_OID) 
    USING STOGROUP BSPMT;
